# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from .raw_client import RawInventoryClient
from ..core.request_options import RequestOptions
from ..types.get_inventory_adjustment_response import GetInventoryAdjustmentResponse
from ..requests.inventory_change import InventoryChangeParams
from ..types.batch_change_inventory_response import BatchChangeInventoryResponse
from ..types.inventory_change_type import InventoryChangeType
from ..types.inventory_state import InventoryState
from ..types.batch_get_inventory_changes_response import BatchGetInventoryChangesResponse
from ..types.batch_get_inventory_counts_response import BatchGetInventoryCountsResponse
from ..core.pagination import SyncPager
from ..types.inventory_change import InventoryChange
from ..core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..types.inventory_count import InventoryCount
from ..types.get_inventory_physical_count_response import GetInventoryPhysicalCountResponse
from ..types.get_inventory_transfer_response import GetInventoryTransferResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..types.get_inventory_count_response import GetInventoryCountResponse
from ..types.get_inventory_changes_response import GetInventoryChangesResponse
from ..core.client_wrapper import AsyncClientWrapper
from .raw_client import AsyncRawInventoryClient
from ..core.pagination import AsyncPager

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class InventoryClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawInventoryClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawInventoryClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawInventoryClient
        """
        return self._raw_client

    def deprecated_get_adjustment(
        self, adjustment_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetInventoryAdjustmentResponse:
        """
        Deprecated version of [RetrieveInventoryAdjustment](api-endpoint:Inventory-RetrieveInventoryAdjustment) after the endpoint URL
        is updated to conform to the standard convention.

        Parameters
        ----------
        adjustment_id : str
            ID of the [InventoryAdjustment](entity:InventoryAdjustment) to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetInventoryAdjustmentResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.inventory.deprecated_get_adjustment(
            adjustment_id="adjustment_id",
        )
        """
        response = self._raw_client.deprecated_get_adjustment(adjustment_id, request_options=request_options)
        return response.data

    def get_adjustment(
        self, adjustment_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetInventoryAdjustmentResponse:
        """
        Returns the [InventoryAdjustment](entity:InventoryAdjustment) object
        with the provided `adjustment_id`.

        Parameters
        ----------
        adjustment_id : str
            ID of the [InventoryAdjustment](entity:InventoryAdjustment) to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetInventoryAdjustmentResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.inventory.get_adjustment(
            adjustment_id="adjustment_id",
        )
        """
        response = self._raw_client.get_adjustment(adjustment_id, request_options=request_options)
        return response.data

    def deprecated_batch_change(
        self,
        *,
        idempotency_key: str,
        changes: typing.Optional[typing.Sequence[InventoryChangeParams]] = OMIT,
        ignore_unchanged_counts: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> BatchChangeInventoryResponse:
        """
        Deprecated version of [BatchChangeInventory](api-endpoint:Inventory-BatchChangeInventory) after the endpoint URL
        is updated to conform to the standard convention.

        Parameters
        ----------
        idempotency_key : str
            A client-supplied, universally unique identifier (UUID) for the
            request.

            See [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency) in the
            [API Development 101](https://developer.squareup.com/docs/buildbasics) section for more
            information.

        changes : typing.Optional[typing.Sequence[InventoryChangeParams]]
            The set of physical counts and inventory adjustments to be made.
            Changes are applied based on the client-supplied timestamp and may be sent
            out of order.

        ignore_unchanged_counts : typing.Optional[bool]
            Indicates whether the current physical count should be ignored if
            the quantity is unchanged since the last physical count. Default: `true`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BatchChangeInventoryResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.inventory.deprecated_batch_change(
            idempotency_key="8fc6a5b0-9fe8-4b46-b46b-2ef95793abbe",
            changes=[
                {
                    "type": "PHYSICAL_COUNT",
                    "physical_count": {
                        "reference_id": "1536bfbf-efed-48bf-b17d-a197141b2a92",
                        "catalog_object_id": "W62UWFY35CWMYGVWK6TWJDNI",
                        "state": "IN_STOCK",
                        "location_id": "C6W5YS5QM06F5",
                        "quantity": "53",
                        "team_member_id": "LRK57NSQ5X7PUD05",
                        "occurred_at": "2016-11-16T22:25:24.878Z",
                    },
                }
            ],
            ignore_unchanged_counts=True,
        )
        """
        response = self._raw_client.deprecated_batch_change(
            idempotency_key=idempotency_key,
            changes=changes,
            ignore_unchanged_counts=ignore_unchanged_counts,
            request_options=request_options,
        )
        return response.data

    def deprecated_batch_get_changes(
        self,
        *,
        catalog_object_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        location_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        types: typing.Optional[typing.Sequence[InventoryChangeType]] = OMIT,
        states: typing.Optional[typing.Sequence[InventoryState]] = OMIT,
        updated_after: typing.Optional[str] = OMIT,
        updated_before: typing.Optional[str] = OMIT,
        cursor: typing.Optional[str] = OMIT,
        limit: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> BatchGetInventoryChangesResponse:
        """
        Deprecated version of [BatchRetrieveInventoryChanges](api-endpoint:Inventory-BatchRetrieveInventoryChanges) after the endpoint URL
        is updated to conform to the standard convention.

        Parameters
        ----------
        catalog_object_ids : typing.Optional[typing.Sequence[str]]
            The filter to return results by `CatalogObject` ID.
            The filter is only applicable when set. The default value is null.

        location_ids : typing.Optional[typing.Sequence[str]]
            The filter to return results by `Location` ID.
            The filter is only applicable when set. The default value is null.

        types : typing.Optional[typing.Sequence[InventoryChangeType]]
            The filter to return results by `InventoryChangeType` values other than `TRANSFER`.
            The default value is `[PHYSICAL_COUNT, ADJUSTMENT]`.

        states : typing.Optional[typing.Sequence[InventoryState]]
            The filter to return `ADJUSTMENT` query results by
            `InventoryState`. This filter is only applied when set.
            The default value is null.

        updated_after : typing.Optional[str]
            The filter to return results with their `calculated_at` value
            after the given time as specified in an RFC 3339 timestamp.
            The default value is the UNIX epoch of (`1970-01-01T00:00:00Z`).

        updated_before : typing.Optional[str]
            The filter to return results with their `created_at` or `calculated_at` value
            strictly before the given time as specified in an RFC 3339 timestamp.
            The default value is the UNIX epoch of (`1970-01-01T00:00:00Z`).

        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this to retrieve the next set of results for the original query.

            See the [Pagination](https://developer.squareup.com/docs/working-with-apis/pagination) guide for more information.

        limit : typing.Optional[int]
            The number of [records](entity:InventoryChange) to return.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BatchGetInventoryChangesResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.inventory.deprecated_batch_get_changes(
            catalog_object_ids=["W62UWFY35CWMYGVWK6TWJDNI"],
            location_ids=["C6W5YS5QM06F5"],
            types=["PHYSICAL_COUNT"],
            states=["IN_STOCK"],
            updated_after="2016-11-01T00:00:00.000Z",
            updated_before="2016-12-01T00:00:00.000Z",
        )
        """
        response = self._raw_client.deprecated_batch_get_changes(
            catalog_object_ids=catalog_object_ids,
            location_ids=location_ids,
            types=types,
            states=states,
            updated_after=updated_after,
            updated_before=updated_before,
            cursor=cursor,
            limit=limit,
            request_options=request_options,
        )
        return response.data

    def deprecated_batch_get_counts(
        self,
        *,
        catalog_object_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        location_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        updated_after: typing.Optional[str] = OMIT,
        cursor: typing.Optional[str] = OMIT,
        states: typing.Optional[typing.Sequence[InventoryState]] = OMIT,
        limit: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> BatchGetInventoryCountsResponse:
        """
        Deprecated version of [BatchRetrieveInventoryCounts](api-endpoint:Inventory-BatchRetrieveInventoryCounts) after the endpoint URL
        is updated to conform to the standard convention.

        Parameters
        ----------
        catalog_object_ids : typing.Optional[typing.Sequence[str]]
            The filter to return results by `CatalogObject` ID.
            The filter is applicable only when set.  The default is null.

        location_ids : typing.Optional[typing.Sequence[str]]
            The filter to return results by `Location` ID.
            This filter is applicable only when set. The default is null.

        updated_after : typing.Optional[str]
            The filter to return results with their `calculated_at` value
            after the given time as specified in an RFC 3339 timestamp.
            The default value is the UNIX epoch of (`1970-01-01T00:00:00Z`).

        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this to retrieve the next set of results for the original query.

            See the [Pagination](https://developer.squareup.com/docs/working-with-apis/pagination) guide for more information.

        states : typing.Optional[typing.Sequence[InventoryState]]
            The filter to return results by `InventoryState`. The filter is only applicable when set.
            Ignored are untracked states of `NONE`, `SOLD`, and `UNLINKED_RETURN`.
            The default is null.

        limit : typing.Optional[int]
            The number of [records](entity:InventoryCount) to return.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BatchGetInventoryCountsResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.inventory.deprecated_batch_get_counts(
            catalog_object_ids=["W62UWFY35CWMYGVWK6TWJDNI"],
            location_ids=["59TNP9SA8VGDA"],
            updated_after="2016-11-16T00:00:00.000Z",
        )
        """
        response = self._raw_client.deprecated_batch_get_counts(
            catalog_object_ids=catalog_object_ids,
            location_ids=location_ids,
            updated_after=updated_after,
            cursor=cursor,
            states=states,
            limit=limit,
            request_options=request_options,
        )
        return response.data

    def batch_create_changes(
        self,
        *,
        idempotency_key: str,
        changes: typing.Optional[typing.Sequence[InventoryChangeParams]] = OMIT,
        ignore_unchanged_counts: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> BatchChangeInventoryResponse:
        """
        Applies adjustments and counts to the provided item quantities.

        On success: returns the current calculated counts for all objects
        referenced in the request.
        On failure: returns a list of related errors.

        Parameters
        ----------
        idempotency_key : str
            A client-supplied, universally unique identifier (UUID) for the
            request.

            See [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency) in the
            [API Development 101](https://developer.squareup.com/docs/buildbasics) section for more
            information.

        changes : typing.Optional[typing.Sequence[InventoryChangeParams]]
            The set of physical counts and inventory adjustments to be made.
            Changes are applied based on the client-supplied timestamp and may be sent
            out of order.

        ignore_unchanged_counts : typing.Optional[bool]
            Indicates whether the current physical count should be ignored if
            the quantity is unchanged since the last physical count. Default: `true`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BatchChangeInventoryResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.inventory.batch_create_changes(
            idempotency_key="8fc6a5b0-9fe8-4b46-b46b-2ef95793abbe",
            changes=[
                {
                    "type": "PHYSICAL_COUNT",
                    "physical_count": {
                        "reference_id": "1536bfbf-efed-48bf-b17d-a197141b2a92",
                        "catalog_object_id": "W62UWFY35CWMYGVWK6TWJDNI",
                        "state": "IN_STOCK",
                        "location_id": "C6W5YS5QM06F5",
                        "quantity": "53",
                        "team_member_id": "LRK57NSQ5X7PUD05",
                        "occurred_at": "2016-11-16T22:25:24.878Z",
                    },
                }
            ],
            ignore_unchanged_counts=True,
        )
        """
        response = self._raw_client.batch_create_changes(
            idempotency_key=idempotency_key,
            changes=changes,
            ignore_unchanged_counts=ignore_unchanged_counts,
            request_options=request_options,
        )
        return response.data

    def batch_get_changes(
        self,
        *,
        catalog_object_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        location_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        types: typing.Optional[typing.Sequence[InventoryChangeType]] = OMIT,
        states: typing.Optional[typing.Sequence[InventoryState]] = OMIT,
        updated_after: typing.Optional[str] = OMIT,
        updated_before: typing.Optional[str] = OMIT,
        cursor: typing.Optional[str] = OMIT,
        limit: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[InventoryChange]:
        """
        Returns historical physical counts and adjustments based on the
        provided filter criteria.

        Results are paginated and sorted in ascending order according their
        `occurred_at` timestamp (oldest first).

        BatchRetrieveInventoryChanges is a catch-all query endpoint for queries
        that cannot be handled by other, simpler endpoints.

        Parameters
        ----------
        catalog_object_ids : typing.Optional[typing.Sequence[str]]
            The filter to return results by `CatalogObject` ID.
            The filter is only applicable when set. The default value is null.

        location_ids : typing.Optional[typing.Sequence[str]]
            The filter to return results by `Location` ID.
            The filter is only applicable when set. The default value is null.

        types : typing.Optional[typing.Sequence[InventoryChangeType]]
            The filter to return results by `InventoryChangeType` values other than `TRANSFER`.
            The default value is `[PHYSICAL_COUNT, ADJUSTMENT]`.

        states : typing.Optional[typing.Sequence[InventoryState]]
            The filter to return `ADJUSTMENT` query results by
            `InventoryState`. This filter is only applied when set.
            The default value is null.

        updated_after : typing.Optional[str]
            The filter to return results with their `calculated_at` value
            after the given time as specified in an RFC 3339 timestamp.
            The default value is the UNIX epoch of (`1970-01-01T00:00:00Z`).

        updated_before : typing.Optional[str]
            The filter to return results with their `created_at` or `calculated_at` value
            strictly before the given time as specified in an RFC 3339 timestamp.
            The default value is the UNIX epoch of (`1970-01-01T00:00:00Z`).

        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this to retrieve the next set of results for the original query.

            See the [Pagination](https://developer.squareup.com/docs/working-with-apis/pagination) guide for more information.

        limit : typing.Optional[int]
            The number of [records](entity:InventoryChange) to return.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[InventoryChange]
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        response = client.inventory.batch_get_changes(
            catalog_object_ids=["W62UWFY35CWMYGVWK6TWJDNI"],
            location_ids=["C6W5YS5QM06F5"],
            types=["PHYSICAL_COUNT"],
            states=["IN_STOCK"],
            updated_after="2016-11-01T00:00:00.000Z",
            updated_before="2016-12-01T00:00:00.000Z",
        )
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        _response = self._raw_client._client_wrapper.httpx_client.request(
            "v2/inventory/changes/batch-retrieve",
            method="POST",
            json={
                "catalog_object_ids": catalog_object_ids,
                "location_ids": location_ids,
                "types": types,
                "states": states,
                "updated_after": updated_after,
                "updated_before": updated_before,
                "cursor": cursor,
                "limit": limit,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    BatchGetInventoryChangesResponse,
                    construct_type(
                        type_=BatchGetInventoryChangesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.batch_get_changes(
                    catalog_object_ids=catalog_object_ids,
                    location_ids=location_ids,
                    types=types,
                    states=states,
                    updated_after=updated_after,
                    updated_before=updated_before,
                    cursor=_parsed_next,
                    limit=limit,
                    request_options=request_options,
                )
                _items = _parsed_response.changes
                return SyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def batch_get_counts(
        self,
        *,
        catalog_object_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        location_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        updated_after: typing.Optional[str] = OMIT,
        cursor: typing.Optional[str] = OMIT,
        states: typing.Optional[typing.Sequence[InventoryState]] = OMIT,
        limit: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[InventoryCount]:
        """
        Returns current counts for the provided
        [CatalogObject](entity:CatalogObject)s at the requested
        [Location](entity:Location)s.

        Results are paginated and sorted in descending order according to their
        `calculated_at` timestamp (newest first).

        When `updated_after` is specified, only counts that have changed since that
        time (based on the server timestamp for the most recent change) are
        returned. This allows clients to perform a "sync" operation, for example
        in response to receiving a Webhook notification.

        Parameters
        ----------
        catalog_object_ids : typing.Optional[typing.Sequence[str]]
            The filter to return results by `CatalogObject` ID.
            The filter is applicable only when set.  The default is null.

        location_ids : typing.Optional[typing.Sequence[str]]
            The filter to return results by `Location` ID.
            This filter is applicable only when set. The default is null.

        updated_after : typing.Optional[str]
            The filter to return results with their `calculated_at` value
            after the given time as specified in an RFC 3339 timestamp.
            The default value is the UNIX epoch of (`1970-01-01T00:00:00Z`).

        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this to retrieve the next set of results for the original query.

            See the [Pagination](https://developer.squareup.com/docs/working-with-apis/pagination) guide for more information.

        states : typing.Optional[typing.Sequence[InventoryState]]
            The filter to return results by `InventoryState`. The filter is only applicable when set.
            Ignored are untracked states of `NONE`, `SOLD`, and `UNLINKED_RETURN`.
            The default is null.

        limit : typing.Optional[int]
            The number of [records](entity:InventoryCount) to return.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[InventoryCount]
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        response = client.inventory.batch_get_counts(
            catalog_object_ids=["W62UWFY35CWMYGVWK6TWJDNI"],
            location_ids=["59TNP9SA8VGDA"],
            updated_after="2016-11-16T00:00:00.000Z",
        )
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        _response = self._raw_client._client_wrapper.httpx_client.request(
            "v2/inventory/counts/batch-retrieve",
            method="POST",
            json={
                "catalog_object_ids": catalog_object_ids,
                "location_ids": location_ids,
                "updated_after": updated_after,
                "cursor": cursor,
                "states": states,
                "limit": limit,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    BatchGetInventoryCountsResponse,
                    construct_type(
                        type_=BatchGetInventoryCountsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.batch_get_counts(
                    catalog_object_ids=catalog_object_ids,
                    location_ids=location_ids,
                    updated_after=updated_after,
                    cursor=_parsed_next,
                    states=states,
                    limit=limit,
                    request_options=request_options,
                )
                _items = _parsed_response.counts
                return SyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def deprecated_get_physical_count(
        self, physical_count_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetInventoryPhysicalCountResponse:
        """
        Deprecated version of [RetrieveInventoryPhysicalCount](api-endpoint:Inventory-RetrieveInventoryPhysicalCount) after the endpoint URL
        is updated to conform to the standard convention.

        Parameters
        ----------
        physical_count_id : str
            ID of the
            [InventoryPhysicalCount](entity:InventoryPhysicalCount) to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetInventoryPhysicalCountResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.inventory.deprecated_get_physical_count(
            physical_count_id="physical_count_id",
        )
        """
        response = self._raw_client.deprecated_get_physical_count(physical_count_id, request_options=request_options)
        return response.data

    def get_physical_count(
        self, physical_count_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetInventoryPhysicalCountResponse:
        """
        Returns the [InventoryPhysicalCount](entity:InventoryPhysicalCount)
        object with the provided `physical_count_id`.

        Parameters
        ----------
        physical_count_id : str
            ID of the
            [InventoryPhysicalCount](entity:InventoryPhysicalCount) to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetInventoryPhysicalCountResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.inventory.get_physical_count(
            physical_count_id="physical_count_id",
        )
        """
        response = self._raw_client.get_physical_count(physical_count_id, request_options=request_options)
        return response.data

    def get_transfer(
        self, transfer_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetInventoryTransferResponse:
        """
        Returns the [InventoryTransfer](entity:InventoryTransfer) object
        with the provided `transfer_id`.

        Parameters
        ----------
        transfer_id : str
            ID of the [InventoryTransfer](entity:InventoryTransfer) to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetInventoryTransferResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.inventory.get_transfer(
            transfer_id="transfer_id",
        )
        """
        response = self._raw_client.get_transfer(transfer_id, request_options=request_options)
        return response.data

    def get(
        self,
        catalog_object_id: str,
        *,
        location_ids: typing.Optional[str] = None,
        cursor: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[InventoryCount]:
        """
        Retrieves the current calculated stock count for a given
        [CatalogObject](entity:CatalogObject) at a given set of
        [Location](entity:Location)s. Responses are paginated and unsorted.
        For more sophisticated queries, use a batch endpoint.

        Parameters
        ----------
        catalog_object_id : str
            ID of the [CatalogObject](entity:CatalogObject) to retrieve.

        location_ids : typing.Optional[str]
            The [Location](entity:Location) IDs to look up as a comma-separated
            list. An empty list queries all locations.

        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this to retrieve the next set of results for the original query.

            See the [Pagination](https://developer.squareup.com/docs/working-with-apis/pagination) guide for more information.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[InventoryCount]
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        response = client.inventory.get(
            catalog_object_id="catalog_object_id",
        )
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        _response = self._raw_client._client_wrapper.httpx_client.request(
            f"v2/inventory/{jsonable_encoder(catalog_object_id)}",
            method="GET",
            params={
                "location_ids": location_ids,
                "cursor": cursor,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    GetInventoryCountResponse,
                    construct_type(
                        type_=GetInventoryCountResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.get(
                    catalog_object_id,
                    location_ids=location_ids,
                    cursor=_parsed_next,
                    request_options=request_options,
                )
                _items = _parsed_response.counts
                return SyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def changes(
        self,
        catalog_object_id: str,
        *,
        location_ids: typing.Optional[str] = None,
        cursor: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[InventoryChange]:
        """
        Returns a set of physical counts and inventory adjustments for the
        provided [CatalogObject](entity:CatalogObject) at the requested
        [Location](entity:Location)s.

        You can achieve the same result by calling [BatchRetrieveInventoryChanges](api-endpoint:Inventory-BatchRetrieveInventoryChanges)
        and having the `catalog_object_ids` list contain a single element of the `CatalogObject` ID.

        Results are paginated and sorted in descending order according to their
        `occurred_at` timestamp (newest first).

        There are no limits on how far back the caller can page. This endpoint can be
        used to display recent changes for a specific item. For more
        sophisticated queries, use a batch endpoint.

        Parameters
        ----------
        catalog_object_id : str
            ID of the [CatalogObject](entity:CatalogObject) to retrieve.

        location_ids : typing.Optional[str]
            The [Location](entity:Location) IDs to look up as a comma-separated
            list. An empty list queries all locations.

        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this to retrieve the next set of results for the original query.

            See the [Pagination](https://developer.squareup.com/docs/working-with-apis/pagination) guide for more information.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[InventoryChange]
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        response = client.inventory.changes(
            catalog_object_id="catalog_object_id",
        )
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        _response = self._raw_client._client_wrapper.httpx_client.request(
            f"v2/inventory/{jsonable_encoder(catalog_object_id)}/changes",
            method="GET",
            params={
                "location_ids": location_ids,
                "cursor": cursor,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    GetInventoryChangesResponse,
                    construct_type(
                        type_=GetInventoryChangesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.changes(
                    catalog_object_id,
                    location_ids=location_ids,
                    cursor=_parsed_next,
                    request_options=request_options,
                )
                _items = _parsed_response.changes
                return SyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncInventoryClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawInventoryClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawInventoryClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawInventoryClient
        """
        return self._raw_client

    async def deprecated_get_adjustment(
        self, adjustment_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetInventoryAdjustmentResponse:
        """
        Deprecated version of [RetrieveInventoryAdjustment](api-endpoint:Inventory-RetrieveInventoryAdjustment) after the endpoint URL
        is updated to conform to the standard convention.

        Parameters
        ----------
        adjustment_id : str
            ID of the [InventoryAdjustment](entity:InventoryAdjustment) to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetInventoryAdjustmentResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.inventory.deprecated_get_adjustment(
                adjustment_id="adjustment_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.deprecated_get_adjustment(adjustment_id, request_options=request_options)
        return response.data

    async def get_adjustment(
        self, adjustment_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetInventoryAdjustmentResponse:
        """
        Returns the [InventoryAdjustment](entity:InventoryAdjustment) object
        with the provided `adjustment_id`.

        Parameters
        ----------
        adjustment_id : str
            ID of the [InventoryAdjustment](entity:InventoryAdjustment) to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetInventoryAdjustmentResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.inventory.get_adjustment(
                adjustment_id="adjustment_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.get_adjustment(adjustment_id, request_options=request_options)
        return response.data

    async def deprecated_batch_change(
        self,
        *,
        idempotency_key: str,
        changes: typing.Optional[typing.Sequence[InventoryChangeParams]] = OMIT,
        ignore_unchanged_counts: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> BatchChangeInventoryResponse:
        """
        Deprecated version of [BatchChangeInventory](api-endpoint:Inventory-BatchChangeInventory) after the endpoint URL
        is updated to conform to the standard convention.

        Parameters
        ----------
        idempotency_key : str
            A client-supplied, universally unique identifier (UUID) for the
            request.

            See [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency) in the
            [API Development 101](https://developer.squareup.com/docs/buildbasics) section for more
            information.

        changes : typing.Optional[typing.Sequence[InventoryChangeParams]]
            The set of physical counts and inventory adjustments to be made.
            Changes are applied based on the client-supplied timestamp and may be sent
            out of order.

        ignore_unchanged_counts : typing.Optional[bool]
            Indicates whether the current physical count should be ignored if
            the quantity is unchanged since the last physical count. Default: `true`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BatchChangeInventoryResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.inventory.deprecated_batch_change(
                idempotency_key="8fc6a5b0-9fe8-4b46-b46b-2ef95793abbe",
                changes=[
                    {
                        "type": "PHYSICAL_COUNT",
                        "physical_count": {
                            "reference_id": "1536bfbf-efed-48bf-b17d-a197141b2a92",
                            "catalog_object_id": "W62UWFY35CWMYGVWK6TWJDNI",
                            "state": "IN_STOCK",
                            "location_id": "C6W5YS5QM06F5",
                            "quantity": "53",
                            "team_member_id": "LRK57NSQ5X7PUD05",
                            "occurred_at": "2016-11-16T22:25:24.878Z",
                        },
                    }
                ],
                ignore_unchanged_counts=True,
            )


        asyncio.run(main())
        """
        response = await self._raw_client.deprecated_batch_change(
            idempotency_key=idempotency_key,
            changes=changes,
            ignore_unchanged_counts=ignore_unchanged_counts,
            request_options=request_options,
        )
        return response.data

    async def deprecated_batch_get_changes(
        self,
        *,
        catalog_object_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        location_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        types: typing.Optional[typing.Sequence[InventoryChangeType]] = OMIT,
        states: typing.Optional[typing.Sequence[InventoryState]] = OMIT,
        updated_after: typing.Optional[str] = OMIT,
        updated_before: typing.Optional[str] = OMIT,
        cursor: typing.Optional[str] = OMIT,
        limit: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> BatchGetInventoryChangesResponse:
        """
        Deprecated version of [BatchRetrieveInventoryChanges](api-endpoint:Inventory-BatchRetrieveInventoryChanges) after the endpoint URL
        is updated to conform to the standard convention.

        Parameters
        ----------
        catalog_object_ids : typing.Optional[typing.Sequence[str]]
            The filter to return results by `CatalogObject` ID.
            The filter is only applicable when set. The default value is null.

        location_ids : typing.Optional[typing.Sequence[str]]
            The filter to return results by `Location` ID.
            The filter is only applicable when set. The default value is null.

        types : typing.Optional[typing.Sequence[InventoryChangeType]]
            The filter to return results by `InventoryChangeType` values other than `TRANSFER`.
            The default value is `[PHYSICAL_COUNT, ADJUSTMENT]`.

        states : typing.Optional[typing.Sequence[InventoryState]]
            The filter to return `ADJUSTMENT` query results by
            `InventoryState`. This filter is only applied when set.
            The default value is null.

        updated_after : typing.Optional[str]
            The filter to return results with their `calculated_at` value
            after the given time as specified in an RFC 3339 timestamp.
            The default value is the UNIX epoch of (`1970-01-01T00:00:00Z`).

        updated_before : typing.Optional[str]
            The filter to return results with their `created_at` or `calculated_at` value
            strictly before the given time as specified in an RFC 3339 timestamp.
            The default value is the UNIX epoch of (`1970-01-01T00:00:00Z`).

        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this to retrieve the next set of results for the original query.

            See the [Pagination](https://developer.squareup.com/docs/working-with-apis/pagination) guide for more information.

        limit : typing.Optional[int]
            The number of [records](entity:InventoryChange) to return.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BatchGetInventoryChangesResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.inventory.deprecated_batch_get_changes(
                catalog_object_ids=["W62UWFY35CWMYGVWK6TWJDNI"],
                location_ids=["C6W5YS5QM06F5"],
                types=["PHYSICAL_COUNT"],
                states=["IN_STOCK"],
                updated_after="2016-11-01T00:00:00.000Z",
                updated_before="2016-12-01T00:00:00.000Z",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.deprecated_batch_get_changes(
            catalog_object_ids=catalog_object_ids,
            location_ids=location_ids,
            types=types,
            states=states,
            updated_after=updated_after,
            updated_before=updated_before,
            cursor=cursor,
            limit=limit,
            request_options=request_options,
        )
        return response.data

    async def deprecated_batch_get_counts(
        self,
        *,
        catalog_object_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        location_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        updated_after: typing.Optional[str] = OMIT,
        cursor: typing.Optional[str] = OMIT,
        states: typing.Optional[typing.Sequence[InventoryState]] = OMIT,
        limit: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> BatchGetInventoryCountsResponse:
        """
        Deprecated version of [BatchRetrieveInventoryCounts](api-endpoint:Inventory-BatchRetrieveInventoryCounts) after the endpoint URL
        is updated to conform to the standard convention.

        Parameters
        ----------
        catalog_object_ids : typing.Optional[typing.Sequence[str]]
            The filter to return results by `CatalogObject` ID.
            The filter is applicable only when set.  The default is null.

        location_ids : typing.Optional[typing.Sequence[str]]
            The filter to return results by `Location` ID.
            This filter is applicable only when set. The default is null.

        updated_after : typing.Optional[str]
            The filter to return results with their `calculated_at` value
            after the given time as specified in an RFC 3339 timestamp.
            The default value is the UNIX epoch of (`1970-01-01T00:00:00Z`).

        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this to retrieve the next set of results for the original query.

            See the [Pagination](https://developer.squareup.com/docs/working-with-apis/pagination) guide for more information.

        states : typing.Optional[typing.Sequence[InventoryState]]
            The filter to return results by `InventoryState`. The filter is only applicable when set.
            Ignored are untracked states of `NONE`, `SOLD`, and `UNLINKED_RETURN`.
            The default is null.

        limit : typing.Optional[int]
            The number of [records](entity:InventoryCount) to return.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BatchGetInventoryCountsResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.inventory.deprecated_batch_get_counts(
                catalog_object_ids=["W62UWFY35CWMYGVWK6TWJDNI"],
                location_ids=["59TNP9SA8VGDA"],
                updated_after="2016-11-16T00:00:00.000Z",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.deprecated_batch_get_counts(
            catalog_object_ids=catalog_object_ids,
            location_ids=location_ids,
            updated_after=updated_after,
            cursor=cursor,
            states=states,
            limit=limit,
            request_options=request_options,
        )
        return response.data

    async def batch_create_changes(
        self,
        *,
        idempotency_key: str,
        changes: typing.Optional[typing.Sequence[InventoryChangeParams]] = OMIT,
        ignore_unchanged_counts: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> BatchChangeInventoryResponse:
        """
        Applies adjustments and counts to the provided item quantities.

        On success: returns the current calculated counts for all objects
        referenced in the request.
        On failure: returns a list of related errors.

        Parameters
        ----------
        idempotency_key : str
            A client-supplied, universally unique identifier (UUID) for the
            request.

            See [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency) in the
            [API Development 101](https://developer.squareup.com/docs/buildbasics) section for more
            information.

        changes : typing.Optional[typing.Sequence[InventoryChangeParams]]
            The set of physical counts and inventory adjustments to be made.
            Changes are applied based on the client-supplied timestamp and may be sent
            out of order.

        ignore_unchanged_counts : typing.Optional[bool]
            Indicates whether the current physical count should be ignored if
            the quantity is unchanged since the last physical count. Default: `true`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BatchChangeInventoryResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.inventory.batch_create_changes(
                idempotency_key="8fc6a5b0-9fe8-4b46-b46b-2ef95793abbe",
                changes=[
                    {
                        "type": "PHYSICAL_COUNT",
                        "physical_count": {
                            "reference_id": "1536bfbf-efed-48bf-b17d-a197141b2a92",
                            "catalog_object_id": "W62UWFY35CWMYGVWK6TWJDNI",
                            "state": "IN_STOCK",
                            "location_id": "C6W5YS5QM06F5",
                            "quantity": "53",
                            "team_member_id": "LRK57NSQ5X7PUD05",
                            "occurred_at": "2016-11-16T22:25:24.878Z",
                        },
                    }
                ],
                ignore_unchanged_counts=True,
            )


        asyncio.run(main())
        """
        response = await self._raw_client.batch_create_changes(
            idempotency_key=idempotency_key,
            changes=changes,
            ignore_unchanged_counts=ignore_unchanged_counts,
            request_options=request_options,
        )
        return response.data

    async def batch_get_changes(
        self,
        *,
        catalog_object_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        location_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        types: typing.Optional[typing.Sequence[InventoryChangeType]] = OMIT,
        states: typing.Optional[typing.Sequence[InventoryState]] = OMIT,
        updated_after: typing.Optional[str] = OMIT,
        updated_before: typing.Optional[str] = OMIT,
        cursor: typing.Optional[str] = OMIT,
        limit: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[InventoryChange]:
        """
        Returns historical physical counts and adjustments based on the
        provided filter criteria.

        Results are paginated and sorted in ascending order according their
        `occurred_at` timestamp (oldest first).

        BatchRetrieveInventoryChanges is a catch-all query endpoint for queries
        that cannot be handled by other, simpler endpoints.

        Parameters
        ----------
        catalog_object_ids : typing.Optional[typing.Sequence[str]]
            The filter to return results by `CatalogObject` ID.
            The filter is only applicable when set. The default value is null.

        location_ids : typing.Optional[typing.Sequence[str]]
            The filter to return results by `Location` ID.
            The filter is only applicable when set. The default value is null.

        types : typing.Optional[typing.Sequence[InventoryChangeType]]
            The filter to return results by `InventoryChangeType` values other than `TRANSFER`.
            The default value is `[PHYSICAL_COUNT, ADJUSTMENT]`.

        states : typing.Optional[typing.Sequence[InventoryState]]
            The filter to return `ADJUSTMENT` query results by
            `InventoryState`. This filter is only applied when set.
            The default value is null.

        updated_after : typing.Optional[str]
            The filter to return results with their `calculated_at` value
            after the given time as specified in an RFC 3339 timestamp.
            The default value is the UNIX epoch of (`1970-01-01T00:00:00Z`).

        updated_before : typing.Optional[str]
            The filter to return results with their `created_at` or `calculated_at` value
            strictly before the given time as specified in an RFC 3339 timestamp.
            The default value is the UNIX epoch of (`1970-01-01T00:00:00Z`).

        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this to retrieve the next set of results for the original query.

            See the [Pagination](https://developer.squareup.com/docs/working-with-apis/pagination) guide for more information.

        limit : typing.Optional[int]
            The number of [records](entity:InventoryChange) to return.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[InventoryChange]
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.inventory.batch_get_changes(
                catalog_object_ids=["W62UWFY35CWMYGVWK6TWJDNI"],
                location_ids=["C6W5YS5QM06F5"],
                types=["PHYSICAL_COUNT"],
                states=["IN_STOCK"],
                updated_after="2016-11-01T00:00:00.000Z",
                updated_before="2016-12-01T00:00:00.000Z",
            )
            async for item in response:
                yield item
            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        _response = await self._raw_client._client_wrapper.httpx_client.request(
            "v2/inventory/changes/batch-retrieve",
            method="POST",
            json={
                "catalog_object_ids": catalog_object_ids,
                "location_ids": location_ids,
                "types": types,
                "states": states,
                "updated_after": updated_after,
                "updated_before": updated_before,
                "cursor": cursor,
                "limit": limit,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    BatchGetInventoryChangesResponse,
                    construct_type(
                        type_=BatchGetInventoryChangesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.batch_get_changes(
                    catalog_object_ids=catalog_object_ids,
                    location_ids=location_ids,
                    types=types,
                    states=states,
                    updated_after=updated_after,
                    updated_before=updated_before,
                    cursor=_parsed_next,
                    limit=limit,
                    request_options=request_options,
                )
                _items = _parsed_response.changes
                return AsyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def batch_get_counts(
        self,
        *,
        catalog_object_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        location_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        updated_after: typing.Optional[str] = OMIT,
        cursor: typing.Optional[str] = OMIT,
        states: typing.Optional[typing.Sequence[InventoryState]] = OMIT,
        limit: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[InventoryCount]:
        """
        Returns current counts for the provided
        [CatalogObject](entity:CatalogObject)s at the requested
        [Location](entity:Location)s.

        Results are paginated and sorted in descending order according to their
        `calculated_at` timestamp (newest first).

        When `updated_after` is specified, only counts that have changed since that
        time (based on the server timestamp for the most recent change) are
        returned. This allows clients to perform a "sync" operation, for example
        in response to receiving a Webhook notification.

        Parameters
        ----------
        catalog_object_ids : typing.Optional[typing.Sequence[str]]
            The filter to return results by `CatalogObject` ID.
            The filter is applicable only when set.  The default is null.

        location_ids : typing.Optional[typing.Sequence[str]]
            The filter to return results by `Location` ID.
            This filter is applicable only when set. The default is null.

        updated_after : typing.Optional[str]
            The filter to return results with their `calculated_at` value
            after the given time as specified in an RFC 3339 timestamp.
            The default value is the UNIX epoch of (`1970-01-01T00:00:00Z`).

        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this to retrieve the next set of results for the original query.

            See the [Pagination](https://developer.squareup.com/docs/working-with-apis/pagination) guide for more information.

        states : typing.Optional[typing.Sequence[InventoryState]]
            The filter to return results by `InventoryState`. The filter is only applicable when set.
            Ignored are untracked states of `NONE`, `SOLD`, and `UNLINKED_RETURN`.
            The default is null.

        limit : typing.Optional[int]
            The number of [records](entity:InventoryCount) to return.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[InventoryCount]
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.inventory.batch_get_counts(
                catalog_object_ids=["W62UWFY35CWMYGVWK6TWJDNI"],
                location_ids=["59TNP9SA8VGDA"],
                updated_after="2016-11-16T00:00:00.000Z",
            )
            async for item in response:
                yield item
            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        _response = await self._raw_client._client_wrapper.httpx_client.request(
            "v2/inventory/counts/batch-retrieve",
            method="POST",
            json={
                "catalog_object_ids": catalog_object_ids,
                "location_ids": location_ids,
                "updated_after": updated_after,
                "cursor": cursor,
                "states": states,
                "limit": limit,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    BatchGetInventoryCountsResponse,
                    construct_type(
                        type_=BatchGetInventoryCountsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.batch_get_counts(
                    catalog_object_ids=catalog_object_ids,
                    location_ids=location_ids,
                    updated_after=updated_after,
                    cursor=_parsed_next,
                    states=states,
                    limit=limit,
                    request_options=request_options,
                )
                _items = _parsed_response.counts
                return AsyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def deprecated_get_physical_count(
        self, physical_count_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetInventoryPhysicalCountResponse:
        """
        Deprecated version of [RetrieveInventoryPhysicalCount](api-endpoint:Inventory-RetrieveInventoryPhysicalCount) after the endpoint URL
        is updated to conform to the standard convention.

        Parameters
        ----------
        physical_count_id : str
            ID of the
            [InventoryPhysicalCount](entity:InventoryPhysicalCount) to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetInventoryPhysicalCountResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.inventory.deprecated_get_physical_count(
                physical_count_id="physical_count_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.deprecated_get_physical_count(
            physical_count_id, request_options=request_options
        )
        return response.data

    async def get_physical_count(
        self, physical_count_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetInventoryPhysicalCountResponse:
        """
        Returns the [InventoryPhysicalCount](entity:InventoryPhysicalCount)
        object with the provided `physical_count_id`.

        Parameters
        ----------
        physical_count_id : str
            ID of the
            [InventoryPhysicalCount](entity:InventoryPhysicalCount) to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetInventoryPhysicalCountResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.inventory.get_physical_count(
                physical_count_id="physical_count_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.get_physical_count(physical_count_id, request_options=request_options)
        return response.data

    async def get_transfer(
        self, transfer_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetInventoryTransferResponse:
        """
        Returns the [InventoryTransfer](entity:InventoryTransfer) object
        with the provided `transfer_id`.

        Parameters
        ----------
        transfer_id : str
            ID of the [InventoryTransfer](entity:InventoryTransfer) to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetInventoryTransferResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.inventory.get_transfer(
                transfer_id="transfer_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.get_transfer(transfer_id, request_options=request_options)
        return response.data

    async def get(
        self,
        catalog_object_id: str,
        *,
        location_ids: typing.Optional[str] = None,
        cursor: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[InventoryCount]:
        """
        Retrieves the current calculated stock count for a given
        [CatalogObject](entity:CatalogObject) at a given set of
        [Location](entity:Location)s. Responses are paginated and unsorted.
        For more sophisticated queries, use a batch endpoint.

        Parameters
        ----------
        catalog_object_id : str
            ID of the [CatalogObject](entity:CatalogObject) to retrieve.

        location_ids : typing.Optional[str]
            The [Location](entity:Location) IDs to look up as a comma-separated
            list. An empty list queries all locations.

        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this to retrieve the next set of results for the original query.

            See the [Pagination](https://developer.squareup.com/docs/working-with-apis/pagination) guide for more information.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[InventoryCount]
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.inventory.get(
                catalog_object_id="catalog_object_id",
            )
            async for item in response:
                yield item
            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        _response = await self._raw_client._client_wrapper.httpx_client.request(
            f"v2/inventory/{jsonable_encoder(catalog_object_id)}",
            method="GET",
            params={
                "location_ids": location_ids,
                "cursor": cursor,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    GetInventoryCountResponse,
                    construct_type(
                        type_=GetInventoryCountResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.get(
                    catalog_object_id,
                    location_ids=location_ids,
                    cursor=_parsed_next,
                    request_options=request_options,
                )
                _items = _parsed_response.counts
                return AsyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def changes(
        self,
        catalog_object_id: str,
        *,
        location_ids: typing.Optional[str] = None,
        cursor: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[InventoryChange]:
        """
        Returns a set of physical counts and inventory adjustments for the
        provided [CatalogObject](entity:CatalogObject) at the requested
        [Location](entity:Location)s.

        You can achieve the same result by calling [BatchRetrieveInventoryChanges](api-endpoint:Inventory-BatchRetrieveInventoryChanges)
        and having the `catalog_object_ids` list contain a single element of the `CatalogObject` ID.

        Results are paginated and sorted in descending order according to their
        `occurred_at` timestamp (newest first).

        There are no limits on how far back the caller can page. This endpoint can be
        used to display recent changes for a specific item. For more
        sophisticated queries, use a batch endpoint.

        Parameters
        ----------
        catalog_object_id : str
            ID of the [CatalogObject](entity:CatalogObject) to retrieve.

        location_ids : typing.Optional[str]
            The [Location](entity:Location) IDs to look up as a comma-separated
            list. An empty list queries all locations.

        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this to retrieve the next set of results for the original query.

            See the [Pagination](https://developer.squareup.com/docs/working-with-apis/pagination) guide for more information.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[InventoryChange]
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.inventory.changes(
                catalog_object_id="catalog_object_id",
            )
            async for item in response:
                yield item
            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        _response = await self._raw_client._client_wrapper.httpx_client.request(
            f"v2/inventory/{jsonable_encoder(catalog_object_id)}/changes",
            method="GET",
            params={
                "location_ids": location_ids,
                "cursor": cursor,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    GetInventoryChangesResponse,
                    construct_type(
                        type_=GetInventoryChangesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.changes(
                    catalog_object_id,
                    location_ids=location_ids,
                    cursor=_parsed_next,
                    request_options=request_options,
                )
                _items = _parsed_response.changes
                return AsyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from ..types.tax_calculation_phase import TaxCalculationPhase
from ..types.tax_inclusion_type import TaxInclusionType


class CatalogTaxParams(typing_extensions.TypedDict):
    """
    A tax applicable to an item.
    """

    name: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The tax's name. This is a searchable attribute for use in applicable query filters, and its value length is of Unicode code points.
    """

    calculation_phase: typing_extensions.NotRequired[TaxCalculationPhase]
    """
    Whether the tax is calculated based on a payment's subtotal or total.
    See [TaxCalculationPhase](#type-taxcalculationphase) for possible values
    """

    inclusion_type: typing_extensions.NotRequired[TaxInclusionType]
    """
    Whether the tax is `ADDITIVE` or `INCLUSIVE`.
    See [TaxInclusionType](#type-taxinclusiontype) for possible values
    """

    percentage: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The percentage of the tax in decimal form, using a `'.'` as the decimal separator and without a `'%'` sign.
    A value of `7.5` corresponds to 7.5%. For a location-specific tax rate, contact the tax authority of the location or a tax consultant.
    """

    applies_to_custom_amounts: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    If `true`, the fee applies to custom amounts entered into the Square Point of Sale
    app that are not associated with a particular `CatalogItem`.
    """

    enabled: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    A Boolean flag to indicate whether the tax is displayed as enabled (`true`) in the Square Point of Sale app or not (`false`).
    """

    applies_to_product_set_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The ID of a `CatalogProductSet` object. If set, the tax is applicable to all products in the product set.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import pydantic
import typing
from .catalog_modifier_override import CatalogModifierOverride
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class CatalogItemModifierListInfo(UncheckedBaseModel):
    """
    Controls how a modifier list is applied to a specific item. This object allows for item-specific customization of modifier list behavior
    and provides the ability to override global modifier list settings.
    """

    modifier_list_id: str = pydantic.Field()
    """
    The ID of the `CatalogModifierList` controlled by this `CatalogModifierListInfo`.
    """

    modifier_overrides: typing.Optional[typing.List[CatalogModifierOverride]] = pydantic.Field(default=None)
    """
    A set of `CatalogModifierOverride` objects that override default modifier settings for this item.
    """

    min_selected_modifiers: typing.Optional[int] = pydantic.Field(default=None)
    """
    The minimum number of modifiers that must be selected from this modifier list.
    Values:
    
    - 0: No selection is required.
    - -1: Default value, the attribute was not set by the client. When `max_selected_modifiers` is
    also -1, use the minimum and maximum selection values set on the `CatalogItemModifierList`.
    - &gt;0: The required minimum modifier selections. This can be larger than the total `CatalogModifiers` when `allow_quantities` is enabled.
    - &lt; -1: Invalid. Treated as no selection required.
    """

    max_selected_modifiers: typing.Optional[int] = pydantic.Field(default=None)
    """
    The maximum number of modifiers that can be selected.
    Values:
    
    - 0: No maximum limit.
    - -1: Default value, the attribute was not set by the client. When `min_selected_modifiers` is
    also -1, use the minimum and maximum selection values set on the `CatalogItemModifierList`.
    - &gt;0: The maximum total modifier selections. This can be larger than the total `CatalogModifiers` when `allow_quantities` is enabled.
    - &lt; -1: Invalid. Treated as no maximum limit.
    """

    enabled: typing.Optional[bool] = pydantic.Field(default=None)
    """
    If `true`, enable this `CatalogModifierList`. The default value is `true`.
    """

    ordinal: typing.Optional[int] = pydantic.Field(default=None)
    """
    The position of this `CatalogItemModifierListInfo` object within the `modifier_list_info` list applied 
    to a `CatalogItem` instance.
    """

    allow_quantities: typing.Optional[typing.Optional[typing.Any]] = None
    is_conversational: typing.Optional[typing.Optional[typing.Any]] = None
    hidden_from_customer_override: typing.Optional[typing.Optional[typing.Any]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from ..types.card_brand import CardBrand
from ..core.serialization import FieldMetadata
import typing
from .address import AddressParams
from ..types.card_type import CardType
from ..types.card_prepaid_type import CardPrepaidType
from ..types.card_co_brand import CardCoBrand
from ..types.card_issuer_alert import CardIssuerAlert


class CardParams(typing_extensions.TypedDict):
    """
    Represents the payment details of a card to be used for payments. These
    details are determined by the payment token generated by Web Payments SDK.
    """

    id: typing_extensions.NotRequired[str]
    """
    Unique ID for this card. Generated by Square.
    """

    card_brand: typing_extensions.NotRequired[CardBrand]
    """
    The card's brand.
    See [CardBrand](#type-cardbrand) for possible values
    """

    last4: typing_extensions.NotRequired[typing_extensions.Annotated[str, FieldMetadata(alias="last_4")]]
    """
    The last 4 digits of the card number.
    """

    exp_month: typing_extensions.NotRequired[typing.Optional[int]]
    """
    The expiration month of the associated card as an integer between 1 and 12.
    """

    exp_year: typing_extensions.NotRequired[typing.Optional[int]]
    """
    The four-digit year of the card's expiration date.
    """

    cardholder_name: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The name of the cardholder.
    """

    billing_address: typing_extensions.NotRequired[AddressParams]
    """
    The billing address for this card. `US` postal codes can be provided as a 5-digit zip code
    or 9-digit ZIP+4 (example: `12345-6789`). For a full list of field meanings by country, see
    [Working with Addresses](https://developer.squareup.com/docs/build-basics/common-data-types/working-with-addresses).
    """

    fingerprint: typing_extensions.NotRequired[str]
    """
    Intended as a Square-assigned identifier, based
    on the card number, to identify the card across multiple locations within a
    single application.
    """

    customer_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    **Required** The ID of a [customer](entity:Customer) to be associated with the card.
    """

    merchant_id: typing_extensions.NotRequired[str]
    """
    The ID of the merchant associated with the card.
    """

    reference_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    An optional user-defined reference ID that associates this card with
    another entity in an external system. For example, a customer ID from an
    external customer management system.
    """

    enabled: typing_extensions.NotRequired[bool]
    """
    Indicates whether or not a card can be used for payments.
    """

    card_type: typing_extensions.NotRequired[CardType]
    """
    The type of the card.
    The Card object includes this field only in response to Payments API calls.
    See [CardType](#type-cardtype) for possible values
    """

    prepaid_type: typing_extensions.NotRequired[CardPrepaidType]
    """
    Indicates whether the card is prepaid or not.
    See [CardPrepaidType](#type-cardprepaidtype) for possible values
    """

    bin: typing_extensions.NotRequired[str]
    """
    The first six digits of the card number, known as the Bank Identification Number (BIN). Only the Payments API
    returns this field.
    """

    version: typing_extensions.NotRequired[int]
    """
    Current version number of the card. Increments with each card update. Requests to update an
    existing Card object will be rejected unless the version in the request matches the current
    version for the Card.
    """

    card_co_brand: typing_extensions.NotRequired[CardCoBrand]
    """
    The card's co-brand if available. For example, an Afterpay virtual card would have a
    co-brand of AFTERPAY.
    See [CardCoBrand](#type-cardcobrand) for possible values
    """

    issuer_alert: typing_extensions.NotRequired[CardIssuerAlert]
    """
    An alert from the issuing bank about the card status. Alerts can indicate whether
    future charges to the card are likely to fail. For more information, see
    [Manage Card on File Declines](https://developer.squareup.com/docs/cards-api/manage-card-on-file-declines).
    
    This field is present only if there's an active issuer alert.
    See [IssuerAlert](#type-issueralert) for possible values
    """

    issuer_alert_at: typing_extensions.NotRequired[str]
    """
    The timestamp of when the current issuer alert was received and processed, in
    RFC 3339 format.
    
    This field is present only if there's an active issuer alert.
    """

    hsa_fsa: typing_extensions.NotRequired[bool]
    """
    Indicates whether the card is linked to a Health Savings Account (HSA) or Flexible
    Spending Account (FSA), based on the card BIN.
    """



# This file was auto-generated by Fern from our API Definition.

import typing

SubscriptionActionType = typing.Union[
    typing.Literal["CANCEL", "PAUSE", "RESUME", "SWAP_PLAN", "CHANGE_BILLING_ANCHOR_DATE"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing


class LoyaltyEventAccumulatePointsParams(typing_extensions.TypedDict):
    """
    Provides metadata when the event `type` is `ACCUMULATE_POINTS`.
    """

    loyalty_program_id: typing_extensions.NotRequired[str]
    """
    The ID of the [loyalty program](entity:LoyaltyProgram).
    """

    points: typing_extensions.NotRequired[typing.Optional[int]]
    """
    The number of points accumulated by the event.
    """

    order_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The ID of the [order](entity:Order) for which the buyer accumulated the points.
    This field is returned only if the Orders API is used to process orders.
    """



# This file was auto-generated by Fern from our API Definition.

import typing

MeasurementUnitUnitType = typing.Union[
    typing.Literal["TYPE_CUSTOM", "TYPE_AREA", "TYPE_LENGTH", "TYPE_VOLUME", "TYPE_WEIGHT", "TYPE_GENERIC"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .loyalty_reward_status import LoyaltyRewardStatus
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class LoyaltyReward(UncheckedBaseModel):
    """
    Represents a contract to redeem loyalty points for a [reward tier](entity:LoyaltyProgramRewardTier) discount. Loyalty rewards can be in an ISSUED, REDEEMED, or DELETED state.
    For more information, see [Manage loyalty rewards](https://developer.squareup.com/docs/loyalty-api/loyalty-rewards).
    """

    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The Square-assigned ID of the loyalty reward.
    """

    status: typing.Optional[LoyaltyRewardStatus] = pydantic.Field(default=None)
    """
    The status of a loyalty reward.
    See [LoyaltyRewardStatus](#type-loyaltyrewardstatus) for possible values
    """

    loyalty_account_id: str = pydantic.Field()
    """
    The Square-assigned ID of the [loyalty account](entity:LoyaltyAccount) to which the reward belongs.
    """

    reward_tier_id: str = pydantic.Field()
    """
    The Square-assigned ID of the [reward tier](entity:LoyaltyProgramRewardTier) used to create the reward.
    """

    points: typing.Optional[int] = pydantic.Field(default=None)
    """
    The number of loyalty points used for the reward.
    """

    order_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The Square-assigned ID of the [order](entity:Order) to which the reward is attached.
    """

    created_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The timestamp when the reward was created, in RFC 3339 format.
    """

    updated_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The timestamp when the reward was last updated, in RFC 3339 format.
    """

    redeemed_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The timestamp when the reward was redeemed, in RFC 3339 format.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class SearchSubscriptionsFilter(UncheckedBaseModel):
    """
    Represents a set of query expressions (filters) to narrow the scope of targeted subscriptions returned by
    the [SearchSubscriptions](api-endpoint:Subscriptions-SearchSubscriptions) endpoint.
    """

    customer_ids: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    A filter to select subscriptions based on the subscribing customer IDs.
    """

    location_ids: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    A filter to select subscriptions based on the location.
    """

    source_names: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    A filter to select subscriptions based on the source application.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class CatalogQueryItemVariationsForItemOptionValues(UncheckedBaseModel):
    """
    The query filter to return the item variations containing the specified item option value IDs.
    """

    item_option_value_ids: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    A set of `CatalogItemOptionValue` IDs to be used to find associated
    `CatalogItemVariation`s. All ItemVariations that contain all of the given
    Item Option Values (in any order) will be returned.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.client_wrapper import SyncClientWrapper
import typing
from ..core.request_options import RequestOptions
from ..core.http_response import HttpResponse
from ..types.list_sites_response import ListSitesResponse
from ..core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper
from ..core.http_response import AsyncHttpResponse


class RawSitesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(self, *, request_options: typing.Optional[RequestOptions] = None) -> HttpResponse[ListSitesResponse]:
        """
        Lists the Square Online sites that belong to a seller. Sites are listed in descending order by the `created_at` date.


        __Note:__ Square Online APIs are publicly available as part of an early access program. For more information, see [Early access program for Square Online APIs](https://developer.squareup.com/docs/online-api#early-access-program-for-square-online-apis).

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ListSitesResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/sites",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ListSitesResponse,
                    construct_type(
                        type_=ListSitesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncRawSitesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[ListSitesResponse]:
        """
        Lists the Square Online sites that belong to a seller. Sites are listed in descending order by the `created_at` date.


        __Note:__ Square Online APIs are publicly available as part of an early access program. For more information, see [Early access program for Square Online APIs](https://developer.squareup.com/docs/online-api#early-access-program-for-square-online-apis).

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ListSitesResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/sites",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ListSitesResponse,
                    construct_type(
                        type_=ListSitesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from ..types.register_domain_response_status import RegisterDomainResponseStatus


class RegisterDomainResponseParams(typing_extensions.TypedDict):
    """
    Defines the fields that are included in the response body of
    a request to the [RegisterDomain](api-endpoint:ApplePay-RegisterDomain) endpoint.

    Either `errors` or `status` are present in a given response (never both).
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """

    status: typing_extensions.NotRequired[RegisterDomainResponseStatus]
    """
    The status of the domain registration.
    
    See [RegisterDomainResponseStatus](entity:RegisterDomainResponseStatus) for possible values.
    See [RegisterDomainResponseStatus](#type-registerdomainresponsestatus) for possible values
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from .money import MoneyParams


class GiftCardActivityImportReversalParams(typing_extensions.TypedDict):
    """
    Represents details about an `IMPORT_REVERSAL` [gift card activity type](entity:GiftCardActivityType).
    """

    amount_money: MoneyParams
    """
    The amount of money cleared from the third-party gift card when 
    the import was reversed.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from .scheduled_shift import ScheduledShiftParams
import typing
from .error import ErrorParams


class RetrieveScheduledShiftResponseParams(typing_extensions.TypedDict):
    """
    Represents a [RetrieveScheduledShift](api-endpoint:Labor-RetrieveScheduledShift) response.
    Either `scheduled_shift` or `errors` is present in the response.
    """

    scheduled_shift: typing_extensions.NotRequired[ScheduledShiftParams]
    """
    The requested scheduled shift.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions


class LoyaltyProgramTerminologyParams(typing_extensions.TypedDict):
    """
    Represents the naming used for loyalty points.
    """

    one: str
    """
    A singular unit for a point (for example, 1 point is called 1 star).
    """

    other: str
    """
    A plural unit for point (for example, 10 points is called 10 stars).
    """



# This file was auto-generated by Fern from our API Definition.

from . import break_types, employee_wages, shifts, team_member_wages, workweek_configs

__all__ = ["break_types", "employee_wages", "shifts", "team_member_wages", "workweek_configs"]



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams


class RetrieveTokenStatusResponseParams(typing_extensions.TypedDict):
    """
    Defines the fields that are included in the response body of
    a request to the `RetrieveTokenStatus` endpoint.
    """

    scopes: typing_extensions.NotRequired[typing.Sequence[str]]
    """
    The list of scopes associated with an access token.
    """

    expires_at: typing_extensions.NotRequired[str]
    """
    The date and time when the `access_token` expires, in RFC 3339 format. Empty if the token never expires.
    """

    client_id: typing_extensions.NotRequired[str]
    """
    The Square-issued application ID associated with the access token. This is the same application ID used to obtain the token.
    """

    merchant_id: typing_extensions.NotRequired[str]
    """
    The ID of the authorizing merchant's business.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
     Any errors that occurred during the request.
    """



# This file was auto-generated by Fern from our API Definition.

import typing

FulfillmentDeliveryDetailsOrderFulfillmentDeliveryDetailsScheduleType = typing.Union[
    typing.Literal["SCHEDULED", "ASAP"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from .custom_attribute import CustomAttributeParams
import typing
from .error import ErrorParams


class RetrieveOrderCustomAttributeResponseParams(typing_extensions.TypedDict):
    """
    Represents a response from getting an order custom attribute.
    """

    custom_attribute: typing_extensions.NotRequired[CustomAttributeParams]
    """
    The retrieved custom attribute. If `with_definition` was set to `true` in the request, the custom attribute definition is returned in the `definition field.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .error import Error
from .cash_drawer_shift_event import CashDrawerShiftEvent
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class ListCashDrawerShiftEventsResponse(UncheckedBaseModel):
    cursor: typing.Optional[str] = pydantic.Field(default=None)
    """
    Opaque cursor for fetching the next page. Cursor is not present in
    the last page of results.
    """

    errors: typing.Optional[typing.List[Error]] = pydantic.Field(default=None)
    """
    Any errors that occurred during the request.
    """

    cash_drawer_shift_events: typing.Optional[typing.List[CashDrawerShiftEvent]] = pydantic.Field(default=None)
    """
    All of the events (payments, refunds, etc.) for a cash drawer during
    the shift.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing


class CatalogQueryRangeParams(typing_extensions.TypedDict):
    """
    The query filter to return the search result whose named attribute values fall between the specified range.
    """

    attribute_name: str
    """
    The name of the attribute to be searched.
    """

    attribute_min_value: typing_extensions.NotRequired[typing.Optional[int]]
    """
    The desired minimum value for the search attribute (inclusive).
    """

    attribute_max_value: typing_extensions.NotRequired[typing.Optional[int]]
    """
    The desired maximum value for the search attribute (inclusive).
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .invoice_recipient import InvoiceRecipient
from .invoice_payment_request import InvoicePaymentRequest
from .invoice_delivery_method import InvoiceDeliveryMethod
from .money import Money
from .invoice_status import InvoiceStatus
from .invoice_accepted_payment_methods import InvoiceAcceptedPaymentMethods
from .invoice_custom_field import InvoiceCustomField
from .invoice_attachment import InvoiceAttachment
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class Invoice(UncheckedBaseModel):
    """
    Stores information about an invoice. You use the Invoices API to create and manage
    invoices. For more information, see [Invoices API Overview](https://developer.squareup.com/docs/invoices-api/overview).
    """

    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The Square-assigned ID of the invoice.
    """

    version: typing.Optional[int] = pydantic.Field(default=None)
    """
    The Square-assigned version number, which is incremented each time an update is committed to the invoice.
    """

    location_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the location that this invoice is associated with. 
    
    If specified in a `CreateInvoice` request, the value must match the `location_id` of the associated order.
    """

    order_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the [order](entity:Order) for which the invoice is created. 
    This field is required when creating an invoice, and the order must be in the `OPEN` state.
    
    To view the line items and other information for the associated order, call the 
    [RetrieveOrder](api-endpoint:Orders-RetrieveOrder) endpoint using the order ID.
    """

    primary_recipient: typing.Optional[InvoiceRecipient] = pydantic.Field(default=None)
    """
    The customer who receives the invoice. This customer data is displayed on the invoice and used by Square to deliver the invoice. 
    
    This field is required to publish an invoice, and it must specify the `customer_id`.
    """

    payment_requests: typing.Optional[typing.List[InvoicePaymentRequest]] = pydantic.Field(default=None)
    """
    The payment schedule for the invoice, represented by one or more payment requests that
    define payment settings, such as amount due and due date. An invoice supports the following payment request combinations:
    - One balance
    - One deposit with one balance
    - 2–12 installments 
    - One deposit with 2–12 installments
    
    This field is required when creating an invoice. It must contain at least one payment request. 
    All payment requests for the invoice must equal the total order amount. For more information, see 
    [Configuring payment requests](https://developer.squareup.com/docs/invoices-api/create-publish-invoices#payment-requests).
    
    Adding `INSTALLMENT` payment requests to an invoice requires an 
    [Invoices Plus subscription](https://developer.squareup.com/docs/invoices-api/overview#invoices-plus-subscription).
    """

    delivery_method: typing.Optional[InvoiceDeliveryMethod] = pydantic.Field(default=None)
    """
    The delivery method that Square uses to send the invoice, reminders, and receipts to
    the customer. After the invoice is published, Square processes the invoice based on the delivery
    method and payment request settings, either immediately or on the `scheduled_at` date, if specified.
    For example, Square might send the invoice or receipt for an automatic payment. For invoices with
    automatic payments, this field must be set to `EMAIL`.
    
    One of the following is required when creating an invoice:
    - (Recommended) This `delivery_method` field. To configure an automatic payment, the
    `automatic_payment_source` field of the payment request is also required.
    - The deprecated `request_method` field of the payment request. Note that `invoice`
    objects returned in responses do not include `request_method`.
    See [InvoiceDeliveryMethod](#type-invoicedeliverymethod) for possible values
    """

    invoice_number: typing.Optional[str] = pydantic.Field(default=None)
    """
    A user-friendly invoice number that is displayed on the invoice. The value is unique within a location.
    If not provided when creating an invoice, Square assigns a value.
    It increments from 1 and is padded with zeros making it 7 characters long
    (for example, 0000001 and 0000002).
    """

    title: typing.Optional[str] = pydantic.Field(default=None)
    """
    The title of the invoice, which is displayed on the invoice.
    """

    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    The description of the invoice, which is displayed on the invoice.
    """

    scheduled_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The timestamp when the invoice is scheduled for processing, in RFC 3339 format.
    After the invoice is published, Square processes the invoice on the specified date,
    according to the delivery method and payment request settings.
    
    If the field is not set, Square processes the invoice immediately after it is published.
    """

    public_url: typing.Optional[str] = pydantic.Field(default=None)
    """
    A temporary link to the Square-hosted payment page where the customer can pay the
    invoice. If the link expires, customers can provide the email address or phone number
    associated with the invoice and request a new link directly from the expired payment page. 
    
    This field is added after the invoice is published and reaches the scheduled date
    (if one is defined).
    """

    next_payment_amount_money: typing.Optional[Money] = pydantic.Field(default=None)
    """
    The current amount due for the invoice. In addition to the
    amount due on the next payment request, this includes any overdue payment amounts.
    """

    status: typing.Optional[InvoiceStatus] = pydantic.Field(default=None)
    """
    The status of the invoice.
    See [InvoiceStatus](#type-invoicestatus) for possible values
    """

    timezone: typing.Optional[str] = pydantic.Field(default=None)
    """
    The time zone used to interpret calendar dates on the invoice, such as `due_date`.
    When an invoice is created, this field is set to the `timezone` specified for the seller
    location. The value cannot be changed.
    
    For example, a payment `due_date` of 2021-03-09 with a `timezone` of America/Los\_Angeles
    becomes overdue at midnight on March 9 in America/Los\_Angeles (which equals a UTC timestamp
    of 2021-03-10T08:00:00Z).
    """

    created_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The timestamp when the invoice was created, in RFC 3339 format.
    """

    updated_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The timestamp when the invoice was last updated, in RFC 3339 format.
    """

    accepted_payment_methods: typing.Optional[InvoiceAcceptedPaymentMethods] = pydantic.Field(default=None)
    """
    The payment methods that customers can use to pay the invoice on the Square-hosted
    invoice page. This setting is independent of any automatic payment requests for the invoice.
    
    This field is required when creating an invoice and must set at least one payment method to `true`.
    """

    custom_fields: typing.Optional[typing.List[InvoiceCustomField]] = pydantic.Field(default=None)
    """
    Additional seller-defined fields that are displayed on the invoice. For more information, see
    [Custom fields](https://developer.squareup.com/docs/invoices-api/overview#custom-fields).
    
    Adding custom fields to an invoice requires an 
    [Invoices Plus subscription](https://developer.squareup.com/docs/invoices-api/overview#invoices-plus-subscription).
    
    Max: 2 custom fields
    """

    subscription_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the [subscription](entity:Subscription) associated with the invoice.
    This field is present only on subscription billing invoices.
    """

    sale_or_service_date: typing.Optional[str] = pydantic.Field(default=None)
    """
    The date of the sale or the date that the service is rendered, in `YYYY-MM-DD` format.
    This field can be used to specify a past or future date which is displayed on the invoice.
    """

    payment_conditions: typing.Optional[str] = pydantic.Field(default=None)
    """
    **France only.** The payment terms and conditions that are displayed on the invoice. For more information, 
    see [Payment conditions](https://developer.squareup.com/docs/invoices-api/overview#payment-conditions).
    
    For countries other than France, Square returns an `INVALID_REQUEST_ERROR` with a `BAD_REQUEST` code and 
    "Payment conditions are not supported for this location's country" detail if this field is included in `CreateInvoice` or `UpdateInvoice` requests.
    """

    store_payment_method_enabled: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Indicates whether to allow a customer to save a credit or debit card as a card on file or a bank transfer as a
    bank account on file. If `true`, Square displays a __Save my card on file__ or __Save my bank on file__ checkbox on the
    invoice payment page. Stored payment information can be used for future automatic payments. The default value is `false`.
    """

    attachments: typing.Optional[typing.List[InvoiceAttachment]] = pydantic.Field(default=None)
    """
    Metadata about the attachments on the invoice. Invoice attachments are managed using the
    [CreateInvoiceAttachment](api-endpoint:Invoices-CreateInvoiceAttachment) and [DeleteInvoiceAttachment](api-endpoint:Invoices-DeleteInvoiceAttachment) endpoints.
    """

    creator_team_member_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the [team member](entity:TeamMember) who created the invoice.
    This field is present only on invoices created in the Square Dashboard or Square Invoices app by a logged-in team member.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions


class CatalogStockConversionParams(typing_extensions.TypedDict):
    """
    Represents the rule of conversion between a stockable [CatalogItemVariation](entity:CatalogItemVariation)
    and a non-stockable sell-by or receive-by `CatalogItemVariation` that
    share the same underlying stock.
    """

    stockable_item_variation_id: str
    """
    References to the stockable [CatalogItemVariation](entity:CatalogItemVariation)
    for this stock conversion. Selling, receiving or recounting the non-stockable `CatalogItemVariation`
    defined with a stock conversion results in adjustments of this stockable `CatalogItemVariation`.
    This immutable field must reference a stockable `CatalogItemVariation`
    that shares the parent [CatalogItem](entity:CatalogItem) of the converted `CatalogItemVariation.`
    """

    stockable_quantity: str
    """
    The quantity of the stockable item variation (as identified by `stockable_item_variation_id`)
    equivalent to the non-stockable item variation quantity (as specified in `nonstockable_quantity`)
    as defined by this stock conversion.  It accepts a decimal number in a string format that can take
    up to 10 digits before the decimal point and up to 5 digits after the decimal point.
    """

    nonstockable_quantity: str
    """
    The converted equivalent quantity of the non-stockable [CatalogItemVariation](entity:CatalogItemVariation)
    in its measurement unit. The `stockable_quantity` value and this `nonstockable_quantity` value together
    define the conversion ratio between stockable item variation and the non-stockable item variation.
    It accepts a decimal number in a string format that can take up to 10 digits before the decimal point
    and up to 5 digits after the decimal point.
    """



# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from .raw_client import RawSnippetsClient
from ..core.request_options import RequestOptions
from ..types.get_snippet_response import GetSnippetResponse
from ..requests.snippet import SnippetParams
from ..types.upsert_snippet_response import UpsertSnippetResponse
from ..types.delete_snippet_response import DeleteSnippetResponse
from ..core.client_wrapper import AsyncClientWrapper
from .raw_client import AsyncRawSnippetsClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class SnippetsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawSnippetsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawSnippetsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawSnippetsClient
        """
        return self._raw_client

    def get(self, site_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> GetSnippetResponse:
        """
        Retrieves your snippet from a Square Online site. A site can contain snippets from multiple snippet applications, but you can retrieve only the snippet that was added by your application.

        You can call [ListSites](api-endpoint:Sites-ListSites) to get the IDs of the sites that belong to a seller.


        __Note:__ Square Online APIs are publicly available as part of an early access program. For more information, see [Early access program for Square Online APIs](https://developer.squareup.com/docs/online-api#early-access-program-for-square-online-apis).

        Parameters
        ----------
        site_id : str
            The ID of the site that contains the snippet.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetSnippetResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.snippets.get(
            site_id="site_id",
        )
        """
        response = self._raw_client.get(site_id, request_options=request_options)
        return response.data

    def upsert(
        self, site_id: str, *, snippet: SnippetParams, request_options: typing.Optional[RequestOptions] = None
    ) -> UpsertSnippetResponse:
        """
        Adds a snippet to a Square Online site or updates the existing snippet on the site.
        The snippet code is appended to the end of the `head` element on every page of the site, except checkout pages. A snippet application can add one snippet to a given site.

        You can call [ListSites](api-endpoint:Sites-ListSites) to get the IDs of the sites that belong to a seller.


        __Note:__ Square Online APIs are publicly available as part of an early access program. For more information, see [Early access program for Square Online APIs](https://developer.squareup.com/docs/online-api#early-access-program-for-square-online-apis).

        Parameters
        ----------
        site_id : str
            The ID of the site where you want to add or update the snippet.

        snippet : SnippetParams
            The snippet for the site.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpsertSnippetResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.snippets.upsert(
            site_id="site_id",
            snippet={"content": "<script>var js = 1;</script>"},
        )
        """
        response = self._raw_client.upsert(site_id, snippet=snippet, request_options=request_options)
        return response.data

    def delete(self, site_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> DeleteSnippetResponse:
        """
        Removes your snippet from a Square Online site.

        You can call [ListSites](api-endpoint:Sites-ListSites) to get the IDs of the sites that belong to a seller.


        __Note:__ Square Online APIs are publicly available as part of an early access program. For more information, see [Early access program for Square Online APIs](https://developer.squareup.com/docs/online-api#early-access-program-for-square-online-apis).

        Parameters
        ----------
        site_id : str
            The ID of the site that contains the snippet.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteSnippetResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.snippets.delete(
            site_id="site_id",
        )
        """
        response = self._raw_client.delete(site_id, request_options=request_options)
        return response.data


class AsyncSnippetsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawSnippetsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawSnippetsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawSnippetsClient
        """
        return self._raw_client

    async def get(self, site_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> GetSnippetResponse:
        """
        Retrieves your snippet from a Square Online site. A site can contain snippets from multiple snippet applications, but you can retrieve only the snippet that was added by your application.

        You can call [ListSites](api-endpoint:Sites-ListSites) to get the IDs of the sites that belong to a seller.


        __Note:__ Square Online APIs are publicly available as part of an early access program. For more information, see [Early access program for Square Online APIs](https://developer.squareup.com/docs/online-api#early-access-program-for-square-online-apis).

        Parameters
        ----------
        site_id : str
            The ID of the site that contains the snippet.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetSnippetResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.snippets.get(
                site_id="site_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.get(site_id, request_options=request_options)
        return response.data

    async def upsert(
        self, site_id: str, *, snippet: SnippetParams, request_options: typing.Optional[RequestOptions] = None
    ) -> UpsertSnippetResponse:
        """
        Adds a snippet to a Square Online site or updates the existing snippet on the site.
        The snippet code is appended to the end of the `head` element on every page of the site, except checkout pages. A snippet application can add one snippet to a given site.

        You can call [ListSites](api-endpoint:Sites-ListSites) to get the IDs of the sites that belong to a seller.


        __Note:__ Square Online APIs are publicly available as part of an early access program. For more information, see [Early access program for Square Online APIs](https://developer.squareup.com/docs/online-api#early-access-program-for-square-online-apis).

        Parameters
        ----------
        site_id : str
            The ID of the site where you want to add or update the snippet.

        snippet : SnippetParams
            The snippet for the site.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpsertSnippetResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.snippets.upsert(
                site_id="site_id",
                snippet={"content": "<script>var js = 1;</script>"},
            )


        asyncio.run(main())
        """
        response = await self._raw_client.upsert(site_id, snippet=snippet, request_options=request_options)
        return response.data

    async def delete(
        self, site_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DeleteSnippetResponse:
        """
        Removes your snippet from a Square Online site.

        You can call [ListSites](api-endpoint:Sites-ListSites) to get the IDs of the sites that belong to a seller.


        __Note:__ Square Online APIs are publicly available as part of an early access program. For more information, see [Early access program for Square Online APIs](https://developer.squareup.com/docs/online-api#early-access-program-for-square-online-apis).

        Parameters
        ----------
        site_id : str
            The ID of the site that contains the snippet.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteSnippetResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.snippets.delete(
                site_id="site_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.delete(site_id, request_options=request_options)
        return response.data



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .catalog_pricing_type import CatalogPricingType
from .money import Money
from .item_variation_location_overrides import ItemVariationLocationOverrides
from .inventory_alert_type import InventoryAlertType
from .catalog_item_option_value_for_item_variation import CatalogItemOptionValueForItemVariation
from .catalog_stock_conversion import CatalogStockConversion
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class CatalogItemVariation(UncheckedBaseModel):
    """
    An item variation, representing a product for sale, in the Catalog object model. Each [item](entity:CatalogItem) must have at least one
    item variation and can have at most 250 item variations.

    An item variation can be sellable, stockable, or both if it has a unit of measure for its count for the sold number of the variation, the stocked
    number of the variation, or both. For example, when a variation representing wine is stocked and sold by the bottle, the variation is both
    stockable and sellable. But when a variation of the wine is sold by the glass, the sold units cannot be used as a measure of the stocked units. This by-the-glass
    variation is sellable, but not stockable. To accurately keep track of the wine's inventory count at any time, the sellable count must be
    converted to stockable count. Typically, the seller defines this unit conversion. For example, 1 bottle equals 5 glasses. The Square API exposes
    the `stockable_conversion` property on the variation to specify the conversion. Thus, when two glasses of the wine are sold, the sellable count
    decreases by 2, and the stockable count automatically decreases by 0.4 bottle according to the conversion.
    """

    item_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the `CatalogItem` associated with this item variation.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The item variation's name. This is a searchable attribute for use in applicable query filters.
    
    Its value has a maximum length of 255 Unicode code points. However, when the parent [item](entity:CatalogItem)
    uses [item options](entity:CatalogItemOption), this attribute is auto-generated, read-only, and can be
    longer than 255 Unicode code points.
    """

    sku: typing.Optional[str] = pydantic.Field(default=None)
    """
    The item variation's SKU, if any. This is a searchable attribute for use in applicable query filters.
    """

    upc: typing.Optional[str] = pydantic.Field(default=None)
    """
    The universal product code (UPC) of the item variation, if any. This is a searchable attribute for use in applicable query filters.
    
    The value of this attribute should be a number of 12-14 digits long.  This restriction is enforced on the Square Seller Dashboard,
    Square Point of Sale or Retail Point of Sale apps, where this attribute shows in the GTIN field. If a non-compliant UPC value is assigned
    to this attribute using the API, the value is not editable on the Seller Dashboard, Square Point of Sale or Retail Point of Sale apps
    unless it is updated to fit the expected format.
    """

    ordinal: typing.Optional[int] = pydantic.Field(default=None)
    """
    The order in which this item variation should be displayed. This value is read-only. On writes, the ordinal
    for each item variation within a parent `CatalogItem` is set according to the item variations's
    position. On reads, the value is not guaranteed to be sequential or unique.
    """

    pricing_type: typing.Optional[CatalogPricingType] = pydantic.Field(default=None)
    """
    Indicates whether the item variation's price is fixed or determined at the time
    of sale.
    See [CatalogPricingType](#type-catalogpricingtype) for possible values
    """

    price_money: typing.Optional[Money] = pydantic.Field(default=None)
    """
    The item variation's price, if fixed pricing is used.
    """

    location_overrides: typing.Optional[typing.List[ItemVariationLocationOverrides]] = pydantic.Field(default=None)
    """
    Per-location price and inventory overrides.
    """

    track_inventory: typing.Optional[bool] = pydantic.Field(default=None)
    """
    If `true`, inventory tracking is active for the variation.
    """

    inventory_alert_type: typing.Optional[InventoryAlertType] = pydantic.Field(default=None)
    """
    Indicates whether the item variation displays an alert when its inventory quantity is less than or equal
    to its `inventory_alert_threshold`.
    See [InventoryAlertType](#type-inventoryalerttype) for possible values
    """

    inventory_alert_threshold: typing.Optional[int] = pydantic.Field(default=None)
    """
    If the inventory quantity for the variation is less than or equal to this value and `inventory_alert_type`
    is `LOW_QUANTITY`, the variation displays an alert in the merchant dashboard.
    
    This value is always an integer.
    """

    user_data: typing.Optional[str] = pydantic.Field(default=None)
    """
    Arbitrary user metadata to associate with the item variation. This attribute value length is of Unicode code points.
    """

    service_duration: typing.Optional[int] = pydantic.Field(default=None)
    """
    If the `CatalogItem` that owns this item variation is of type
    `APPOINTMENTS_SERVICE`, then this is the duration of the service in milliseconds. For
    example, a 30 minute appointment would have the value `1800000`, which is equal to
    30 (minutes) * 60 (seconds per minute) * 1000 (milliseconds per second).
    """

    available_for_booking: typing.Optional[bool] = pydantic.Field(default=None)
    """
    If the `CatalogItem` that owns this item variation is of type
    `APPOINTMENTS_SERVICE`, a bool representing whether this service is available for booking.
    """

    item_option_values: typing.Optional[typing.List[CatalogItemOptionValueForItemVariation]] = pydantic.Field(
        default=None
    )
    """
    List of item option values associated with this item variation. Listed
    in the same order as the item options of the parent item.
    """

    measurement_unit_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    ID of the ‘CatalogMeasurementUnit’ that is used to measure the quantity
    sold of this item variation. If left unset, the item will be sold in
    whole quantities.
    """

    sellable: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether this variation can be sold. The inventory count of a sellable variation indicates
    the number of units available for sale. When a variation is both stockable and sellable,
    its sellable inventory count can be smaller than or equal to its stockable count.
    """

    stockable: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether stock is counted directly on this variation (TRUE) or only on its components (FALSE).
    When a variation is both stockable and sellable, the inventory count of a stockable variation keeps track of the number of units of this variation in stock
    and is not an indicator of the number of units of the variation that can be sold.
    """

    image_ids: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    The IDs of images associated with this `CatalogItemVariation` instance.
    These images will be shown to customers in Square Online Store.
    """

    team_member_ids: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Tokens of employees that can perform the service represented by this variation. Only valid for
    variations of type `APPOINTMENTS_SERVICE`.
    """

    stockable_conversion: typing.Optional[CatalogStockConversion] = pydantic.Field(default=None)
    """
    The unit conversion rule, as prescribed by the [CatalogStockConversion](entity:CatalogStockConversion) type,
    that describes how this non-stockable (i.e., sellable/receivable) item variation is converted
    to/from the stockable item variation sharing the same parent item. With the stock conversion,
    you can accurately track inventory when an item variation is sold in one unit, but stocked in
    another unit.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions


class QrCodeOptionsParams(typing_extensions.TypedDict):
    """
    Fields to describe the action that displays QR-Codes.
    """

    title: str
    """
    The title text to display in the QR code flow on the Terminal.
    """

    body: str
    """
    The body text to display in the QR code flow on the Terminal.
    """

    barcode_contents: str
    """
    The text representation of the data to show in the QR code
    as UTF8-encoded data.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from .money import MoneyParams
import typing
from .processing_fee import ProcessingFeeParams
from .card_payment_details import CardPaymentDetailsParams
from .cash_payment_details import CashPaymentDetailsParams
from .bank_account_payment_details import BankAccountPaymentDetailsParams
from .external_payment_details import ExternalPaymentDetailsParams
from .digital_wallet_details import DigitalWalletDetailsParams
from .buy_now_pay_later_details import BuyNowPayLaterDetailsParams
from .square_account_details import SquareAccountDetailsParams
from .risk_evaluation import RiskEvaluationParams
from .address import AddressParams
from .device_details import DeviceDetailsParams
from .application_details import ApplicationDetailsParams
from .offline_payment_details import OfflinePaymentDetailsParams


class PaymentParams(typing_extensions.TypedDict):
    """
    Represents a payment processed by the Square API.
    """

    id: typing_extensions.NotRequired[str]
    """
    A unique ID for the payment.
    """

    created_at: typing_extensions.NotRequired[str]
    """
    The timestamp of when the payment was created, in RFC 3339 format.
    """

    updated_at: typing_extensions.NotRequired[str]
    """
    The timestamp of when the payment was last updated, in RFC 3339 format.
    """

    amount_money: typing_extensions.NotRequired[MoneyParams]
    """
    The amount processed for this payment, not including `tip_money`.
    
    The amount is specified in the smallest denomination of the applicable currency (for example,
    US dollar amounts are specified in cents). For more information, see
    [Working with Monetary Amounts](https://developer.squareup.com/docs/build-basics/working-with-monetary-amounts).
    """

    tip_money: typing_extensions.NotRequired[MoneyParams]
    """
    The amount designated as a tip. 
    
    This amount is specified in the smallest denomination of the applicable currency (for example,
    US dollar amounts are specified in cents). For more information, see
    [Working with Monetary Amounts](https://developer.squareup.com/docs/build-basics/working-with-monetary-amounts).
    """

    total_money: typing_extensions.NotRequired[MoneyParams]
    """
    The total amount for the payment, including `amount_money` and `tip_money`.
    This amount is specified in the smallest denomination of the applicable currency (for example,
    US dollar amounts are specified in cents). For more information, see
    [Working with Monetary Amounts](https://developer.squareup.com/docs/build-basics/working-with-monetary-amounts).
    """

    app_fee_money: typing_extensions.NotRequired[MoneyParams]
    """
    The amount the developer is taking as a fee for facilitating the payment on behalf
    of the seller. This amount is specified in the smallest denomination of the applicable currency
    (for example, US dollar amounts are specified in cents). For more information,
    see [Take Payments and Collect Fees](https://developer.squareup.com/docs/payments-api/take-payments-and-collect-fees).
    
    The amount cannot be more than 90% of the `total_money` value.
    
    To set this field, `PAYMENTS_WRITE_ADDITIONAL_RECIPIENTS` OAuth permission is required.
    For more information, see [Permissions](https://developer.squareup.com/docs/payments-api/take-payments-and-collect-fees#permissions).
    """

    approved_money: typing_extensions.NotRequired[MoneyParams]
    """
    The amount of money approved for this payment. This value may change if Square chooses to
    obtain reauthorization as part of a call to [UpdatePayment](api-endpoint:Payments-UpdatePayment).
    """

    processing_fee: typing_extensions.NotRequired[typing.Sequence[ProcessingFeeParams]]
    """
    The processing fees and fee adjustments assessed by Square for this payment.
    """

    refunded_money: typing_extensions.NotRequired[MoneyParams]
    """
    The total amount of the payment refunded to date. 
    
    This amount is specified in the smallest denomination of the applicable currency (for example,
    US dollar amounts are specified in cents).
    """

    status: typing_extensions.NotRequired[str]
    """
    Indicates whether the payment is APPROVED, PENDING, COMPLETED, CANCELED, or FAILED.
    """

    delay_duration: typing_extensions.NotRequired[str]
    """
    The duration of time after the payment's creation when Square automatically applies the
    `delay_action` to the payment. This automatic `delay_action` applies only to payments that
    do not reach a terminal state (COMPLETED, CANCELED, or FAILED) before the `delay_duration`
    time period.
    
    This field is specified as a time duration, in RFC 3339 format.
    
    Notes:
    This feature is only supported for card payments.
    
    Default:
    
    - Card-present payments: "PT36H" (36 hours) from the creation time.
    - Card-not-present payments: "P7D" (7 days) from the creation time.
    """

    delay_action: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The action to be applied to the payment when the `delay_duration` has elapsed.
    
    Current values include `CANCEL` and `COMPLETE`.
    """

    delayed_until: typing_extensions.NotRequired[str]
    """
    The read-only timestamp of when the `delay_action` is automatically applied,
    in RFC 3339 format.
    
    Note that this field is calculated by summing the payment's `delay_duration` and `created_at`
    fields. The `created_at` field is generated by Square and might not exactly match the
    time on your local machine.
    """

    source_type: typing_extensions.NotRequired[str]
    """
    The source type for this payment.
    
    Current values include `CARD`, `BANK_ACCOUNT`, `WALLET`, `BUY_NOW_PAY_LATER`, `SQUARE_ACCOUNT`,
    `CASH` and `EXTERNAL`. For information about these payment source types,
    see [Take Payments](https://developer.squareup.com/docs/payments-api/take-payments).
    """

    card_details: typing_extensions.NotRequired[CardPaymentDetailsParams]
    """
    Details about a card payment. These details are only populated if the source_type is `CARD`.
    """

    cash_details: typing_extensions.NotRequired[CashPaymentDetailsParams]
    """
    Details about a cash payment. These details are only populated if the source_type is `CASH`.
    """

    bank_account_details: typing_extensions.NotRequired[BankAccountPaymentDetailsParams]
    """
    Details about a bank account payment. These details are only populated if the source_type is `BANK_ACCOUNT`.
    """

    external_details: typing_extensions.NotRequired[ExternalPaymentDetailsParams]
    """
    Details about an external payment. The details are only populated 
    if the `source_type` is `EXTERNAL`.
    """

    wallet_details: typing_extensions.NotRequired[DigitalWalletDetailsParams]
    """
    Details about an wallet payment. The details are only populated 
    if the `source_type` is `WALLET`.
    """

    buy_now_pay_later_details: typing_extensions.NotRequired[BuyNowPayLaterDetailsParams]
    """
    Details about a Buy Now Pay Later payment. The details are only populated
    if the `source_type` is `BUY_NOW_PAY_LATER`. For more information, see 
    [Afterpay Payments](https://developer.squareup.com/docs/payments-api/take-payments/afterpay-payments).
    """

    square_account_details: typing_extensions.NotRequired[SquareAccountDetailsParams]
    """
    Details about a Square Account payment. The details are only populated
    if the `source_type` is `SQUARE_ACCOUNT`.
    """

    location_id: typing_extensions.NotRequired[str]
    """
    The ID of the location associated with the payment.
    """

    order_id: typing_extensions.NotRequired[str]
    """
    The ID of the order associated with the payment.
    """

    reference_id: typing_extensions.NotRequired[str]
    """
    An optional ID that associates the payment with an entity in
    another system.
    """

    customer_id: typing_extensions.NotRequired[str]
    """
    The ID of the customer associated with the payment. If the ID is 
    not provided in the `CreatePayment` request that was used to create the `Payment`, 
    Square may use information in the request 
    (such as the billing and shipping address, email address, and payment source) 
    to identify a matching customer profile in the Customer Directory. 
    If found, the profile ID is used. If a profile is not found, the 
    API attempts to create an 
    [instant profile](https://developer.squareup.com/docs/customers-api/what-it-does#instant-profiles). 
    If the API cannot create an 
    instant profile (either because the seller has disabled it or the 
    seller's region prevents creating it), this field remains unset. Note that 
    this process is asynchronous and it may take some time before a 
    customer ID is added to the payment.
    """

    employee_id: typing_extensions.NotRequired[str]
    """
    __Deprecated__: Use `Payment.team_member_id` instead.
    
    An optional ID of the employee associated with taking the payment.
    """

    team_member_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    An optional ID of the [TeamMember](entity:TeamMember) associated with taking the payment.
    """

    refund_ids: typing_extensions.NotRequired[typing.Sequence[str]]
    """
    A list of `refund_id`s identifying refunds for the payment.
    """

    risk_evaluation: typing_extensions.NotRequired[RiskEvaluationParams]
    """
    Provides information about the risk associated with the payment, as determined by Square.
    This field is present for payments to sellers that have opted in to receive risk
    evaluations.
    """

    terminal_checkout_id: typing_extensions.NotRequired[str]
    """
    An optional ID for a Terminal checkout that is associated with the payment.
    """

    buyer_email_address: typing_extensions.NotRequired[str]
    """
    The buyer's email address.
    """

    billing_address: typing_extensions.NotRequired[AddressParams]
    """
    The buyer's billing address.
    """

    shipping_address: typing_extensions.NotRequired[AddressParams]
    """
    The buyer's shipping address.
    """

    note: typing_extensions.NotRequired[str]
    """
    An optional note to include when creating a payment.
    """

    statement_description_identifier: typing_extensions.NotRequired[str]
    """
    Additional payment information that gets added to the customer's card statement
    as part of the statement description.
    
    Note that the `statement_description_identifier` might get truncated on the statement description
    to fit the required information including the Square identifier (SQ *) and the name of the
    seller taking the payment.
    """

    capabilities: typing_extensions.NotRequired[typing.Sequence[str]]
    """
    Actions that can be performed on this payment:
    - `EDIT_AMOUNT_UP` - The payment amount can be edited up.
    - `EDIT_AMOUNT_DOWN` - The payment amount can be edited down.
    - `EDIT_TIP_AMOUNT_UP` - The tip amount can be edited up.
    - `EDIT_TIP_AMOUNT_DOWN` - The tip amount can be edited down.
    - `EDIT_DELAY_ACTION` - The delay_action can be edited.
    """

    receipt_number: typing_extensions.NotRequired[str]
    """
    The payment's receipt number.
    The field is missing if a payment is canceled.
    """

    receipt_url: typing_extensions.NotRequired[str]
    """
    The URL for the payment's receipt.
    The field is only populated for COMPLETED payments.
    """

    device_details: typing_extensions.NotRequired[DeviceDetailsParams]
    """
    Details about the device that took the payment.
    """

    application_details: typing_extensions.NotRequired[ApplicationDetailsParams]
    """
    Details about the application that took the payment.
    """

    is_offline_payment: typing_extensions.NotRequired[bool]
    """
    Whether or not this payment was taken offline.
    """

    offline_payment_details: typing_extensions.NotRequired[OfflinePaymentDetailsParams]
    """
    Additional information about the payment if it was taken offline.
    """

    version_token: typing_extensions.NotRequired[typing.Optional[str]]
    """
    Used for optimistic concurrency. This opaque token identifies a specific version of the
    `Payment` object.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .order_entry import OrderEntryParams
from .order import OrderParams
from .error import ErrorParams


class SearchOrdersResponseParams(typing_extensions.TypedDict):
    """
    Either the `order_entries` or `orders` field is set, depending on whether
    `return_entries` is set on the [SearchOrdersRequest](api-endpoint:Orders-SearchOrders).
    """

    order_entries: typing_extensions.NotRequired[typing.Sequence[OrderEntryParams]]
    """
    A list of [OrderEntries](entity:OrderEntry) that fit the query
    conditions. The list is populated only if `return_entries` is set to `true` in the request.
    """

    orders: typing_extensions.NotRequired[typing.Sequence[OrderParams]]
    """
    A list of
    [Order](entity:Order) objects that match the query conditions. The list is populated only if
    `return_entries` is set to `false` in the request.
    """

    cursor: typing_extensions.NotRequired[str]
    """
    The pagination cursor to be used in a subsequent request. If unset,
    this is the final response.
    For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    [Errors](entity:Error) encountered during the search.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from .checkout_merchant_settings_payment_methods_payment_method import (
    CheckoutMerchantSettingsPaymentMethodsPaymentMethodParams,
)
from .checkout_merchant_settings_payment_methods_afterpay_clearpay import (
    CheckoutMerchantSettingsPaymentMethodsAfterpayClearpayParams,
)


class CheckoutMerchantSettingsPaymentMethodsParams(typing_extensions.TypedDict):
    apple_pay: typing_extensions.NotRequired[CheckoutMerchantSettingsPaymentMethodsPaymentMethodParams]
    google_pay: typing_extensions.NotRequired[CheckoutMerchantSettingsPaymentMethodsPaymentMethodParams]
    cash_app: typing_extensions.NotRequired[CheckoutMerchantSettingsPaymentMethodsPaymentMethodParams]
    afterpay_clearpay: typing_extensions.NotRequired[CheckoutMerchantSettingsPaymentMethodsAfterpayClearpayParams]



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .money import Money
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class OrderLineItemAppliedServiceCharge(UncheckedBaseModel):
    uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    A unique ID that identifies the applied service charge only within this order.
    """

    service_charge_uid: str = pydantic.Field()
    """
    The `uid` of the service charge that the applied service charge represents. It must
    reference a service charge present in the `order.service_charges` field.
    
    This field is immutable. To change which service charges apply to a line item,
    delete and add a new `OrderLineItemAppliedServiceCharge`.
    """

    applied_money: typing.Optional[Money] = pydantic.Field(default=None)
    """
    The amount of money applied by the service charge to the line item.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions


class BookingCustomAttributeDeleteRequestParams(typing_extensions.TypedDict):
    """
    Represents an individual delete request in a [BulkDeleteBookingCustomAttributes](api-endpoint:BookingCustomAttributes-BulkDeleteBookingCustomAttributes)
    request. An individual request contains a booking ID, the custom attribute to delete, and an optional idempotency key.
    """

    booking_id: str
    """
    The ID of the target [booking](entity:Booking).
    """

    key: str
    """
    The key of the custom attribute to delete. This key must match the `key` of a
    custom attribute definition in the Square seller account. If the requesting application is not
    the definition owner, you must use the qualified key.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .order_quantity_unit import OrderQuantityUnitParams
from ..types.order_line_item_item_type import OrderLineItemItemType
from .order_return_line_item_modifier import OrderReturnLineItemModifierParams
from .order_line_item_applied_tax import OrderLineItemAppliedTaxParams
from .order_line_item_applied_discount import OrderLineItemAppliedDiscountParams
from .money import MoneyParams
from .order_line_item_applied_service_charge import OrderLineItemAppliedServiceChargeParams


class OrderReturnLineItemParams(typing_extensions.TypedDict):
    """
    The line item being returned in an order.
    """

    uid: typing_extensions.NotRequired[typing.Optional[str]]
    """
    A unique ID for this return line-item entry.
    """

    source_line_item_uid: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The `uid` of the line item in the original sale order.
    """

    name: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The name of the line item.
    """

    quantity: str
    """
    The quantity returned, formatted as a decimal number.
    For example, `"3"`.
    
    Line items with a `quantity_unit` can have non-integer quantities.
    For example, `"1.70000"`.
    """

    quantity_unit: typing_extensions.NotRequired[OrderQuantityUnitParams]
    """
    The unit and precision that this return line item's quantity is measured in.
    """

    note: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The note of the return line item.
    """

    catalog_object_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The [CatalogItemVariation](entity:CatalogItemVariation) ID applied to this return line item.
    """

    catalog_version: typing_extensions.NotRequired[typing.Optional[int]]
    """
    The version of the catalog object that this line item references.
    """

    variation_name: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The name of the variation applied to this return line item.
    """

    item_type: typing_extensions.NotRequired[OrderLineItemItemType]
    """
    The type of line item: an itemized return, a non-itemized return (custom amount),
    or the return of an unactivated gift card sale.
    See [OrderLineItemItemType](#type-orderlineitemitemtype) for possible values
    """

    return_modifiers: typing_extensions.NotRequired[typing.Optional[typing.Sequence[OrderReturnLineItemModifierParams]]]
    """
    The [CatalogModifier](entity:CatalogModifier)s applied to this line item.
    """

    applied_taxes: typing_extensions.NotRequired[typing.Optional[typing.Sequence[OrderLineItemAppliedTaxParams]]]
    """
    The list of references to `OrderReturnTax` entities applied to the return line item. Each
    `OrderLineItemAppliedTax` has a `tax_uid` that references the `uid` of a top-level
    `OrderReturnTax` applied to the return line item. On reads, the applied amount
    is populated.
    """

    applied_discounts: typing_extensions.NotRequired[
        typing.Optional[typing.Sequence[OrderLineItemAppliedDiscountParams]]
    ]
    """
    The list of references to `OrderReturnDiscount` entities applied to the return line item. Each
    `OrderLineItemAppliedDiscount` has a `discount_uid` that references the `uid` of a top-level
    `OrderReturnDiscount` applied to the return line item. On reads, the applied amount
    is populated.
    """

    base_price_money: typing_extensions.NotRequired[MoneyParams]
    """
    The base price for a single unit of the line item.
    """

    variation_total_price_money: typing_extensions.NotRequired[MoneyParams]
    """
    The total price of all item variations returned in this line item.
    The price is calculated as `base_price_money` multiplied by `quantity` and
    does not include modifiers.
    """

    gross_return_money: typing_extensions.NotRequired[MoneyParams]
    """
    The gross return amount of money calculated as (item base price + modifiers price) * quantity.
    """

    total_tax_money: typing_extensions.NotRequired[MoneyParams]
    """
    The total amount of tax money to return for the line item.
    """

    total_discount_money: typing_extensions.NotRequired[MoneyParams]
    """
    The total amount of discount money to return for the line item.
    """

    total_money: typing_extensions.NotRequired[MoneyParams]
    """
    The total amount of money to return for this line item.
    """

    applied_service_charges: typing_extensions.NotRequired[
        typing.Optional[typing.Sequence[OrderLineItemAppliedServiceChargeParams]]
    ]
    """
    The list of references to `OrderReturnServiceCharge` entities applied to the return
    line item. Each `OrderLineItemAppliedServiceCharge` has a `service_charge_uid` that
    references the `uid` of a top-level `OrderReturnServiceCharge` applied to the return line
    item. On reads, the applied amount is populated.
    """

    total_service_charge_money: typing_extensions.NotRequired[MoneyParams]
    """
    The total amount of apportioned service charge money to return for the line item.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import pydantic
from .money import Money
from ..core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class ChargeRequestAdditionalRecipient(UncheckedBaseModel):
    """
    Represents an additional recipient (other than the merchant) entitled to a portion of the tender.
    Support is currently limited to USD, CAD and GBP currencies
    """

    location_id: str = pydantic.Field()
    """
    The location ID for a recipient (other than the merchant) receiving a portion of the tender.
    """

    description: str = pydantic.Field()
    """
    The description of the additional recipient.
    """

    amount_money: Money = pydantic.Field()
    """
    The amount of money distributed to the recipient.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

InvoiceStatus = typing.Union[
    typing.Literal[
        "DRAFT",
        "UNPAID",
        "SCHEDULED",
        "PARTIALLY_PAID",
        "PAID",
        "PARTIALLY_REFUNDED",
        "REFUNDED",
        "CANCELED",
        "FAILED",
        "PAYMENT_PENDING",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class OrderLineItemPricingBlocklistsBlockedTax(UncheckedBaseModel):
    """
    A tax to block from applying to a line item. The tax must be
    identified by either `tax_uid` or `tax_catalog_object_id`, but not both.
    """

    uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    A unique ID of the `BlockedTax` within the order.
    """

    tax_uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    The `uid` of the tax that should be blocked. Use this field to block
    ad hoc taxes. For catalog, taxes use the `tax_catalog_object_id` field.
    """

    tax_catalog_object_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The `catalog_object_id` of the tax that should be blocked.
    Use this field to block catalog taxes. For ad hoc taxes, use the
    `tax_uid` field.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .payment_link import PaymentLinkParams
from .payment_link_related_resources import PaymentLinkRelatedResourcesParams


class CreatePaymentLinkResponseParams(typing_extensions.TypedDict):
    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """

    payment_link: typing_extensions.NotRequired[PaymentLinkParams]
    """
    The created payment link.
    """

    related_resources: typing_extensions.NotRequired[PaymentLinkRelatedResourcesParams]
    """
    The list of related objects.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .dispute_evidence import DisputeEvidenceParams


class GetDisputeEvidenceResponseParams(typing_extensions.TypedDict):
    """
    Defines the fields in a `RetrieveDisputeEvidence` response.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """

    evidence: typing_extensions.NotRequired[DisputeEvidenceParams]
    """
    Metadata about the dispute evidence file.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from ..types.risk_evaluation_risk_level import RiskEvaluationRiskLevel


class RiskEvaluationParams(typing_extensions.TypedDict):
    """
    Represents fraud risk information for the associated payment.

    When you take a payment through Square's Payments API (using the `CreatePayment`
    endpoint), Square evaluates it and assigns a risk level to the payment. Sellers
    can use this information to determine the course of action (for example,
    provide the goods/services or refund the payment).
    """

    created_at: typing_extensions.NotRequired[str]
    """
    The timestamp when payment risk was evaluated, in RFC 3339 format.
    """

    risk_level: typing_extensions.NotRequired[RiskEvaluationRiskLevel]
    """
    The risk level associated with the payment
    See [RiskEvaluationRiskLevel](#type-riskevaluationrisklevel) for possible values
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from .invoice_filter import InvoiceFilterParams
import typing_extensions
from .invoice_sort import InvoiceSortParams


class InvoiceQueryParams(typing_extensions.TypedDict):
    """
    Describes query criteria for searching invoices.
    """

    filter: InvoiceFilterParams
    """
    Query filters to apply in searching invoices. 
    For more information, see [Search for invoices](https://developer.squareup.com/docs/invoices-api/retrieve-list-search-invoices#search-invoices).
    """

    sort: typing_extensions.NotRequired[InvoiceSortParams]
    """
    Describes the sort order for the search result.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
from .catalog_category import CatalogCategory
from .catalog_item import CatalogItem
from .catalog_item_option import CatalogItemOption
from .catalog_modifier_list import CatalogModifierList
from .catalog_object_category import CatalogObjectCategory
from .catalog_object_item import CatalogObjectItem
from .catalog_object_item_option import CatalogObjectItemOption
from .catalog_object_modifier_list import CatalogObjectModifierList
from .catalog_object_subscription_plan import CatalogObjectSubscriptionPlan
from .catalog_subscription_plan import CatalogSubscriptionPlan
import typing
from .catalog_object import CatalogObject
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class CatalogObjectBatch(UncheckedBaseModel):
    """
    A batch of catalog objects.
    """

    objects: typing.List[CatalogObject] = pydantic.Field()
    """
    A list of CatalogObjects belonging to this batch.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import pydantic
from .custom_attribute import CustomAttribute
import typing
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class BulkUpsertMerchantCustomAttributesRequestMerchantCustomAttributeUpsertRequest(UncheckedBaseModel):
    """
    Represents an individual upsert request in a [BulkUpsertMerchantCustomAttributes](api-endpoint:MerchantCustomAttributes-BulkUpsertMerchantCustomAttributes)
    request. An individual request contains a merchant ID, the custom attribute to create or update,
    and an optional idempotency key.
    """

    merchant_id: str = pydantic.Field()
    """
    The ID of the target [merchant](entity:Merchant).
    """

    custom_attribute: CustomAttribute = pydantic.Field()
    """
    The custom attribute to create or update, with following fields:
    - `key`. This key must match the `key` of a custom attribute definition in the Square seller
    account. If the requesting application is not the definition owner, you must provide the qualified key.
    - `value`. This value must conform to the `schema` specified by the definition.
    For more information, see [Supported data types](https://developer.squareup.com/docs/devtools/customattributes/overview#supported-data-types).
    - The version field must match the current version of the custom attribute definition to enable
    [optimistic concurrency](https://developer.squareup.com/docs/build-basics/common-api-patterns/optimistic-concurrency)
    If this is not important for your application, version can be set to -1. For any other values, the request fails with a BAD_REQUEST error.
    """

    idempotency_key: typing.Optional[str] = pydantic.Field(default=None)
    """
    A unique identifier for this individual upsert request, used to ensure idempotency.
    For more information, see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from .money import MoneyParams
import typing_extensions


class CashPaymentDetailsParams(typing_extensions.TypedDict):
    """
    Stores details about a cash payment. Contains only non-confidential information. For more information, see
    [Take Cash Payments](https://developer.squareup.com/docs/payments-api/take-payments/cash-payments).
    """

    buyer_supplied_money: MoneyParams
    """
    The amount and currency of the money supplied by the buyer.
    """

    change_back_money: typing_extensions.NotRequired[MoneyParams]
    """
    The amount of change due back to the buyer. 
    This read-only field is calculated
    from the `amount_money` and `buyer_supplied_money` fields.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class UpdateCatalogImageRequest(UncheckedBaseModel):
    idempotency_key: str = pydantic.Field()
    """
    A unique string that identifies this UpdateCatalogImage request.
    Keys can be any valid string but must be unique for every UpdateCatalogImage request.
    
    See [Idempotency keys](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency) for more information.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

TeamMemberAssignedLocationsAssignmentType = typing.Union[
    typing.Literal["ALL_CURRENT_AND_FUTURE_LOCATIONS", "EXPLICIT_LOCATIONS"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing
from .catalog_object import CatalogObjectParams


class CatalogObjectBatchParams(typing_extensions.TypedDict):
    """
    A batch of catalog objects.
    """

    objects: typing.Sequence[CatalogObjectParams]
    """
    A list of CatalogObjects belonging to this batch.
    """
